angular.module('condition').controller("PropertyCondition", function ($scope, i18nService, profileService, definitionService) {
    var choices = [
        {
            "id": "equals",
            "name": "comparisonOperator.equals",
            "appliesTo": ["email", "string", "integer"]
        }, {
            "id": "notEquals",
            "name": "comparisonOperator.notEquals",
            "appliesTo": ["email", "string", "integer"]
        }, {
            "id": "lessThan",
            "name": "comparisonOperator.lessThan",
            "appliesTo": ["integer"]
        }, {
            "id": "greaterThan",
            "name": "comparisonOperator.greaterThan",
            "appliesTo": ["integer"]
        }, {
            "id": "lessThanOrEqualTo",
            "name": "comparisonOperator.lessThanOrEqualTo",
            "appliesTo": ["integer"]
        }, {
            "id": "greaterThanOrEqualTo",
            "name": "comparisonOperator.greaterThanOrEqualTo",
            "appliesTo": ["integer"]
        }, {
            "id": "lessThanOrEqualTo",
            "name": "comparisonOperator.before",
            "appliesTo": ["date"]
        }, {
            "id": "greaterThanOrEqualTo",
            "name": "comparisonOperator.after",
            "appliesTo": ["date"]
        }, {
            "id": "isDay",
            "name": "comparisonOperator.isDay",
            "appliesTo": ["date"]
        }, {
            "id": "isNotDay",
            "name": "comparisonOperator.isNotDay",
            "appliesTo": ["date"]
        }, {
            "id": "between",
            "name": "comparisonOperator.between",
            "appliesTo": ["date", "integer"],
            "isMultivalued": true
        }, {
            "id": "startsWith",
            "name": "comparisonOperator.startsWith",
            "appliesTo": ["email", "string"]
        }, {
            "id": "endsWith",
            "name": "comparisonOperator.endsWith",
            "appliesTo": ["email", "string"]
        }, {
            "id": "matchesRegex",
            "name": "comparisonOperator.matchesRegex",
            "appliesTo": ["email", "string"]
        }, {
            "id": "contains",
            "name": "comparisonOperator.contains",
            "appliesTo": ["email", "string"]
        }, {
            "id": "inContains",
            "name": "comparisonOperator.inContains",
            "appliesTo": ["email", "string"],
            "isMultivalued": true
        },{
            "id": "notContains",
            "name": "comparisonOperator.notContains",
            "appliesTo": ["email", "string"]
        }, {
            "id": "exists",
            "name": "comparisonOperator.exists",
            "withoutValues": true
        }, {
            "id": "missing",
            "name": "comparisonOperator.missing",
            "withoutValues": true
        }, {
            "id": "in",
            "name": "comparisonOperator.in",
            "appliesTo": ["email", "string", "integer", "multivalued"],
            "isMultivalued": true
        }, {
            "id": "notIn",
            "name": "comparisonOperator.notIn",
            "appliesTo": ["email", "string", "integer"],
            "isMultivalued": true
        }, {
            "id": "hasSomeOf",
            "name": "comparisonOperator.hasSomeOf",
            "appliesTo": ["multivalued"],
            "isMultivalued": true
        }, {
            "id": "hasNoneOf",
            "name": "comparisonOperator.hasNoneOf",
            "appliesTo": ["multivalued"],
            "isMultivalued": true
        }, {
            "id": "all",
            "name": "comparisonOperator.all",
            "appliesTo": ["multivalued"],
            "isMultivalued": true
        }];

    function initComparisonOperators() {
        if ($scope.propertyValueType) {
            if($scope.propertyValueType.multivalued){
                $scope.operators = _.filter(choices, function (choice) {
                    return _.contains(choice.appliesTo, "multivalued") || !choice.appliesTo;
                });
            }else {
                $scope.operators = _.filter(choices, function (choice) {
                    return _.contains(choice.appliesTo, $scope.propertyValueType.type) || !choice.appliesTo;
                });
            }

            if(!$scope.ref.parameterValues.comparisonOperator) {
                $scope.ref.parameterValues.comparisonOperator = $scope.operators[0].id;
            }
            getOpObj();
        } else {
            $scope.operators = choices;
        }
        $scope.isOperatorWithValues = false;
        hasValues();
    }

    var getOpObj = function() {
        $scope.comparisonOperatorObj = _.find(choices, function (op) {
            return op.id === $scope.ref.parameterValues.comparisonOperator;
        });
    };

    function initPropertyValueType() {
        if (!$scope.propertyCondition.selectedPropertyName) {
            return;
        }
        if ($scope.propertyCondition.propertyNameChoiceListValues) {
            var choice = _.find($scope.propertyCondition.propertyNameChoiceListValues, function (value) {
                if (!$scope.propertyCondition.predefinedProperty) {
                    return value.metadata.id === $scope.propertyCondition.propertyValueOriginalName;
                } else {
                    return value.metadata.id === $scope.ref.parameterValues.propertyName;
                }
            });
            if (choice) {
                if (!$scope.propertyValueType) {
                    $scope.propertyValueType = {};
                }
                $scope.propertyValueType = {
                    type: choice.type,
                    multivalued: choice.multivalued
                };
                return;
            }
        }
        $scope.propertyValueType = null;
    }

    function resetParameterValues() {
        delete $scope.ref.parameterValues.propertyValue;
        delete $scope.ref.parameterValues.propertyValueInteger;
        delete $scope.ref.parameterValues.propertyValueDate;
        delete $scope.ref.parameterValues.propertyValues;
        delete $scope.ref.parameterValues.propertyValuesInteger;
        delete $scope.ref.parameterValues.propertyValuesDate;
        delete $scope.ref.parameterValues.propertyValueDateExpr;
        delete $scope.ref.parameterValues.propertyValuesDateExpr;

        getOpObj();

        if ($scope.propertyValueType) {
            if ($scope.comparisonOperatorObj && $scope.comparisonOperatorObj.isMultivalued) {
                if ($scope.propertyValueType.type == 'date') {
                    $scope.ref.parameterValues.propertyValuesDate = [undefined];
                    if($scope.comparisonOperatorObj.id == 'between') {
                        $scope.ref.parameterValues.propertyValuesDate[1] = undefined;
                    }
                } else if ($scope.propertyValueType.type == 'integer') {
                    $scope.ref.parameterValues.propertyValuesInteger = [undefined];
                    if($scope.comparisonOperatorObj.id == 'between') {
                        $scope.ref.parameterValues.propertyValuesInteger[1] = undefined;
                    }
                } else if ($scope.propertyValueType.type != 'none') {
                    $scope.ref.parameterValues.propertyValues = [''];
                    if($scope.comparisonOperatorObj.id == 'between') {
                        $scope.ref.parameterValues.propertyValues[1] = '';
                    }
                }
            } else if ($scope.comparisonOperatorObj && $scope.comparisonOperatorObj.id != 'exists' && $scope.comparisonOperatorObj.id != 'missing') {
                if ($scope.propertyValueType.type == 'date') {
                    $scope.ref.parameterValues.propertyValueDate = undefined;
                } else if ($scope.propertyValueType.type == 'integer') {
                    $scope.ref.parameterValues.propertyValueInteger = undefined;
                } else if ($scope.propertyValueType.type != 'none') {
                    $scope.ref.parameterValues.propertyValue = '';
                }
            }

        }
    }

    $scope.propertyCondition = {};

    function processData(data) {
        $scope.propertyNameChoiceListValuesIds = [];
        $scope.propertyNameChoiceListValues = data;
        $scope.propertyNameChoiceListValues.forEach(function (prop) {
            prop.metadata.id = "properties." + prop.metadata.id;
            $scope.propertyNameChoiceListValuesIds.push(prop.metadata.id)
        });

        $scope.propertyCondition.propertyNameChoiceListValues = $scope.propertyNameChoiceListValues.slice(0);
        $scope.propertyCondition.propertyNameChoiceListValues.push({
            metadata: {
                id: 'user-specified-property-string',
                name: i18nService.message('condition.propertyCondition.othertext')
            },
            type: 'string',
            multivalued: false
        });
        $scope.propertyCondition.propertyNameChoiceListValues.push({
            metadata: {
                id: 'user-specified-property-integer',
                name: i18nService.message('condition.propertyCondition.othernumeric')
            },
            type: 'integer',
            multivalued: false
        });
        $scope.propertyCondition.propertyNameChoiceListValues.push({
            metadata: {
                id: 'user-specified-property-date',
                name: i18nService.message('condition.propertyCondition.otherdate')
            },
            type: 'date',
            multivalued: false
        });

        $scope.propertyCondition.predefinedProperty = $scope.ref.parameterValues.propertyName == null || $scope.propertyNameChoiceListValuesIds.indexOf($scope.ref.parameterValues.propertyName) > -1;
        if ($scope.propertyCondition.predefinedProperty) {
            $scope.propertyCondition.selectedPropertyName = $scope.ref.parameterValues.propertyName;
        } else {
            if ($scope.ref.parameterValues.propertyValueInteger || $scope.ref.parameterValues.propertyValuesInteger) {
                $scope.propertyCondition.selectedPropertyName = 'user-specified-property-integer';
            } else if ($scope.ref.parameterValues.propertyValueDate || $scope.ref.parameterValues.propertyValuesDate) {
                $scope.propertyCondition.selectedPropertyName = 'user-specified-property-date';
            } else {
                $scope.propertyCondition.selectedPropertyName = 'user-specified-property-string';
            }
        }

        initPropertyValueType();
        initComparisonOperators();
    }

    var retrieveData = function() {
        if ($scope.conditionType.id == "profilePropertyCondition") {
            profileService.getPropertyTypesByTag("profileProperties", true).success(processData);
        } else if ($scope.conditionType.id == "sessionPropertyCondition") {
            profileService.getPropertyTypesByTag("sessionProperties", true).success(processData);
        } else {
            processData([]);
        }
    };

    retrieveData();

    //Event broadcasted when  a variant is selected.
    $scope.$on('eventVariantSelected', function(event, data) {
        retrieveData();
    });

    $scope.changePropertyName = function() {
        var selectedPropertyName = $scope.propertyCondition.selectedPropertyName;
        $scope.propertyCondition.predefinedProperty = selectedPropertyName != 'user-specified-property-string' && selectedPropertyName != 'user-specified-property-date' && selectedPropertyName != 'user-specified-property-integer';
        if ($scope.propertyCondition.predefinedProperty) {
            $scope.ref.parameterValues.propertyName = $scope.propertyCondition.selectedPropertyName;
        } else {
            $scope.ref.parameterValues.propertyName = '';
            $scope.propertyCondition.propertyValueOriginalName = $scope.propertyCondition.selectedPropertyName
        }
        initPropertyValueType();
        initComparisonOperators();
        $scope.ref.parameterValues.comparisonOperator = null;
    };

    $scope.changeComparisonOperator = function() {
        resetParameterValues();
        hasValues();
    };

    var hasValues = function(){
        var choice = _.find(choices, function (obj) {
            return obj.id === $scope.ref.parameterValues.comparisonOperator;
        });
        if(choice){
            $scope.isOperatorWithValues = !choice.withoutValues;
        }
    };

    $scope.requiredLabel = i18nService.message('label.required');
});
