angular.module('manageGoals', ['ngResource', 'chart.js', 'wemServices', 'wemDirectives', 'ui.bootstrap', 'picker', 'i18n', 'datatables', 'underscore', 'timeline', 'wemModals', 'ui-notification', 'manageMostVisited'])
    .run(['$rootScope', '$location', 'jcrService', 'genericModal', 'i18nService', function ($rootScope, $location, jcrService, genericModal, i18nService) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };

        $rootScope.getGoalViewTemplate = function (goal) {
            if (goal && goal.metadata && goal.metadata.tags && goal.metadata.tags.length > 0) {
                return ManagersContext.localTemplatesPath + "/goals/goal-view-" + goal.metadata.tags[0] + ".html";
            }
        };

        $rootScope.getGoalLinksView = function () {
            return ManagersContext.localTemplatesPath + "/goals/goal-widget-links.html";
        };

        var history = [];

        $rootScope.$on('$routeChangeSuccess', function () {
            history.push($location.$$path);
        });

        $rootScope.back = function () {
            var prevUrl = history.length > 1 ? history.splice(-2)[0] : "/";
            $location.path(prevUrl);
        };

        $rootScope.validateGoalPaths = function(paths, callback, async, noModal, errorPaths) {
            if(!errorPaths) {
                errorPaths = [];
            }

            if (paths && paths.length > 0) {
                var path = paths[0];
                paths.shift();
                if(async) {
                    jcrService.doGetOnPath("live", null, path).success(function() {
                        $rootScope.validateGoalPaths(paths, callback, async, noModal, errorPaths);
                    }).error(function() {
                        errorPaths.push(path);
                        $rootScope.validateGoalPaths(paths, callback, async, noModal, errorPaths);
                    })
                } else {
                    var result = jcrService.doGetOnPathSync("live", null, path);
                    if(!result) {
                        errorPaths.push(path);
                    }
                    $rootScope.validateGoalPaths(paths, callback, async, noModal, errorPaths);
                }
            } else {
                if(errorPaths.length > 0 && !noModal) {
                    genericModal.newAlertModal({
                        title: i18nService.message("label.warning"),
                        bodyMessage: i18nService.format("serverSettings.manageGoals.validationPathAndOk", errorPaths.join(", "))
                    }, function () {
                        if (callback) {
                            callback(errorPaths);
                        }
                    });
                } else {
                    if(callback) {
                        callback(errorPaths);
                    }
                }
            }
        };
    }])

    .directive("goalWidget", ["goalService", "ruleService", "genericModal", "i18nService", function (goalService, ruleService, genericModal, i18nService) {
        return {
            restrict: "E",
            scope: {
                goal: '=',
                tagId: '='
            },
            template: '<div ng-include="templateUrl"></div>',
            controller: ['$rootScope', '$scope', "$route", "goalService", "$location", function ($rootScope, $scope, $route, goalService, $location) {

                // get report
                var query = {
                    aggregate: {
                        type: "date",
                        property: "timeStamp",
                        parameters: {
                            interval: "1d"
                        }
                    }
                };
                goalService.getGoalReportAggregated($scope.goal.id, query).success(function (reportData) {
                    $scope.report = reportData.globalStats;
                    $scope.report.conversion = ManagersUtils.getPercentage($scope.report.conversionRate);
                    $scope.doughnutLabels = ["Success", "Views"];
                    $scope.doughnutData = [$scope.report.conversion, 100 - $scope.report.conversion];
                    var doughnutColor = ManagersUtils.getDoughnutColor($scope.report.conversion);
                    $scope.doughnutColor = [doughnutColor, "#eeeeee"];
                    $scope.doughnutOpts = {
                        animation: true,
                        responsive: true,
                        showTooltips: false,
                        percentageInnerCutout: 60,
                        segmentShowStroke: false,
                        onAnimationComplete: function () {
                            var constant = 114;
                            var fontsize = (this.chart.height / constant).toFixed(2);
                            this.chart.ctx.font = fontsize + "em Arial";
                            this.chart.ctx.fillStyle = doughnutColor;
                            this.chart.ctx.textBaseline = "middle";
                            var tpercentage = $scope.report.conversion + "%";
                            var textWidth = this.chart.ctx.measureText(tpercentage).width;
                            var txtPosx = Math.round((this.chart.width - textWidth) / 2);
                            this.chart.ctx.fillText(tpercentage, txtPosx, this.chart.height / 2);
                        }
                    };

                    // compute trend
                    if (reportData.split && reportData.split.length > 0) {
                        var currentWeek = moment().startOf("week");
                        var weekOneBeforeCurrent = moment().startOf("week").subtract(1, 'weeks');
                        var weekTwoBeforeCurrent = moment().startOf("week").subtract(2, 'weeks');
                        var result = {
                            startCountBefore: 0,
                            startCountBeforeTwo: 0,
                            targetCountBefore: 0,
                            targetCountBeforeTwo: 0
                        };
                        angular.forEach(reportData.split, function (v, k) {
                            if (v && v.key) {
                                var thisMoment = moment(v.key);
                                if (thisMoment.isBetween(weekTwoBeforeCurrent, currentWeek)) {
                                    if (thisMoment.isAfter(weekOneBeforeCurrent)) {
                                        result.startCountBefore = result.startCountBefore + v.startCount;
                                        result.targetCountBefore = result.targetCountBefore + v.targetCount;
                                    } else {
                                        result.startCountBeforeTwo = result.startCountBeforeTwo + v.startCount;
                                        result.targetCountBeforeTwo = result.targetCountBeforeTwo + v.targetCount;
                                    }
                                }
                            }
                        });
                        result.conversionBefore = result.startCountBefore > 0 ? result.targetCountBefore / result.startCountBefore : 0;
                        result.conversionBeforeTwo = result.startCountBeforeTwo > 0 ? result.targetCountBeforeTwo / result.startCountBeforeTwo : 0;
                        if (result.conversionBefore > result.conversionBeforeTwo) {
                            $scope.progressionArrow = "fa fa-arrow-up";
                        } else if (result.conversionBefore == result.conversionBeforeTwo) {
                            $scope.progressionArrow = "fa fa-minus";
                        } else {
                            $scope.progressionArrow = "fa fa-arrow-down";
                        }
                    }
                });

                // resolve template
                $scope.templateUrl = ManagersContext.localTemplatesPath + "/goals/goal-widget-"+$scope.tagId+".html";

                $scope.goalLinksTmpl = $rootScope.getGoalLinksView();

                $scope.deleteGoal = function (id) {
                    genericModal.newAlertModal({
                        title: i18nService.message("serverSettings.manageWem.delete"),
                        bodyMessage: i18nService.message("serverSettings.manageGoals.delete.confirm")
                    }, function () {
                        goalService.deleteGoal(id).success(function (data) {
                            ruleService.deleteRule(goalService.getAddToListRulePrefix()+id);
                            $route.reload();
                        });
                    });
                };
            }]
        };
    }])
    .directive("goalReportTable", ["goalService", "queryService", "$http", 'geonamesService', '$modal',  function (goalService, queryService, $http, geonamesService) {
        return {
            restrict: "E",
            scope: {
                goal: '=',
                properties: '=',
                data: '=ngModel',
                type: "@",
                queryRowHook: "@",
                queryHook: "@",
                tableColumnsHook: "@"
            },
            templateUrl: ManagersContext.localTemplatesPath + "/goals/goal-report-table.html",
            controller: ['$scope', "goalService", "i18nService", "segmentService", "DTOptionsBuilder", "DTColumnBuilder", "$modal", function ($scope, goalService, i18nService, segmentService, DTOptionsBuilder, DTColumnBuilder, $modal) {
                $scope.facets = {};
                $scope.split = [];
                $scope.ranges = {};

                $scope.actionsHtml = function (data, type, full, meta) {
                    // data.selectedProperty.
                    return "<button title=\"" + i18nService.message("serverSettings.manageSegments.manageMembers") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"openManagement(" + data.key + ")\" ><span class=\"md md-profiles\"></span></button>";
                };

                $scope.openManagement = function () {
                    var dialogModal = {
                        backdrop: true,
                        keyboard: true,
                        templateUrl: ManagersContext.localTemplatesPath + '/sessions/sessions-list.html', // Url du template HTML
                        // templateUrl: ManagersContext.localTemplatesPath + '/sessions/sessions-list.html', // Url du template HTML
                        controller: 'sessionModalController', //  function ($scope, $modalInstance, id)
                        resolve: {
                            scopeParent: function () {
                                return $scope; //On passe à la fenêtre modal une référence vers le scope parent
                            },
                            currentProperty: function () {
                                return $scope.data.selectedProperty.itemId;
                            }
                        }
                    };
                    $modal.open(dialogModal);
                };

                $scope.setDtInstance = function (val) {
                    $scope.dtInstance = val;
                };

                $scope.isNotFacetedProp = function (value, index) {
                    return !$scope.facets[value.id];
                };

                $scope.showSegment = function (value, index) {
                    return value.itemType != 'segments' || (value.itemType == 'segments' && $scope.isQualitativeReport());
                };

                $scope.showRangeCheckbox = function () {
                    return $scope.data && $scope.data.selectedProperty && $scope._isPropertyRanged($scope.data.selectedProperty);
                };

                $scope.showDateInterval = function () {
                    return $scope.data && $scope.data.selectedProperty && $scope.data.selectedProperty.type == "date";
                };

                $scope.getFacetDisplay = function (facet) {
                    return facet.property.metadata.name + " : " + (facet.value.displayableKey ? ((facet.value.displayableKey.length > 50)?facet.value.displayableKey.substring(0, 50) + '...':facet.value.displayableKey) : facet.value.key);
                };

                $scope.addFacet = function (selectedProperty, rowValue) {
                    var facet = {
                        property: angular.copy(selectedProperty),
                        value: angular.copy(rowValue),
                        conditions: []
                    };

                    // resolve associated conditions to selectedProperty and rowValue
                    var isSession = facet.property.itemType == "sessions";

                    var propertyName = facet.property.metadata.id;
                    if (facet.property.itemType == "sessions") {
                        propertyName = "properties." + propertyName;
                    } else if (facet.property.itemType == "profiles") {
                        propertyName = "profile.properties." + propertyName;
                    } else if (propertyName == "segments") {
                        propertyName = "profile.segments"
                    }
                    if ($scope._isPropertyRanged(facet.property) && $scope.data.useRanges) {
                        var range = $scope.ranges[facet.property.metadata.id] ? $scope.ranges[facet.property.metadata.id][facet.value.key] : undefined;
                        if (range) {
                            if (range.from) {
                                facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, "greaterThan", facet.property.type, false, range.from));
                            }
                            if (range.to) {
                                facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, "lessThan", facet.property.type, false, range.to));
                            }
                        }
                    } else if (facet.property.type == "date") {
                        // handle date range
                        for (var i = 0; i < $scope.split.length; i++) {
                            var split = $scope.split[i];
                            if (split.key == facet.value.key) {
                                if (i == $scope.split.length - 1) {
                                    facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, "greaterThan", facet.property.type, false,
                                        $scope._getSelectedIntervalAsDate(split.key)));
                                } else {
                                    facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, "between", facet.property.type, true,
                                        [$scope._getSelectedIntervalAsDate(split.key), $scope._getSelectedIntervalAsDate($scope.split[i + 1].key)]));
                                }
                                break;
                            }
                        }
                    } else {
                        if (facet.value.missing) {
                            facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, "missing", facet.property.type, false, null));
                        } else {
                            facet.conditions.push($scope._getPropertyCondition(isSession, propertyName, "equals", facet.property.type, false, facet.value.key));
                        }
                    }


                    $scope.facets[selectedProperty.metadata.id] = facet;
                    var filteredProperties = $scope._getFilteredProperties();
                    $scope.data.selectedProperty = filteredProperties && filteredProperties.length > 0 ? filteredProperties[0] : undefined;
                };

                $scope.removeFacet = function (facet) {
                    if (facet && $scope.facets[facet.property.metadata.id]) {
                        delete $scope.facets[facet.property.metadata.id];
                        $scope.data.selectedProperty = $scope._getProperty(facet.property.metadata.id);
                    }
                };

                $scope.isQualitativeReport = function () {
                    return $scope.type == "qualitative";
                };

                $scope.isQuantitativeReport = function () {
                    return $scope.type == "quantitative";
                };

                $scope.getTableLabel = function () {
                    return i18nService.message("serverSettings.manageGoals.report.title." + $scope.type);
                };

                $scope._getProperty = function (propertyId) {
                    for (var propIndex in $scope.properties) {
                        if (propertyId == $scope.properties[propIndex].metadata.id) {
                            return $scope.properties[propIndex];
                        }
                    }
                    return null;
                };

                $scope._getFilteredProperties = function () {
                    return $scope.properties.filter(function (property) {
                        return !$scope.facets[property.metadata.id];
                    });
                };

                $scope._isPropertyRanged = function (property) {
                    return (property.numericRanges && property.numericRanges.length > 0) ||
                        (property.dateRanges && property.dateRanges.length > 0) ||
                        (property.ipRanges && property.ipRanges.length > 0);
                };

                $scope._getPropertyCondition = function (isSession, propertyName, operator, type, multiple, propertyValue) {
                    var condition = {
                        "type": isSession ? "sessionPropertyCondition" : "profilePropertyCondition",
                        "parameterValues": {
                            "propertyName": propertyName,
                            "comparisonOperator": operator
                        }
                    };

                    switch (type) {
                        case "integer":
                            if (multiple) {
                                condition.parameterValues.propertyValuesInteger = propertyValue;
                            } else {
                                condition.parameterValues.propertyValueInteger = propertyValue;
                            }
                            break;
                        case "date":
                            if (multiple) {
                                condition.parameterValues.propertyValuesDate = propertyValue;
                            } else {
                                condition.parameterValues.propertyValueDate = propertyValue;
                            }
                            break;
                        default :
                            condition.parameterValues.propertyValue = propertyValue;
                    }

                    return condition;
                };

                $scope._getI18nForRange = function (type, to, from) {
                    var t,f;
                    if (to != 'now') {
                        t = to;
                    }
                    if (from != 'now') {
                        f = from;
                    }
                    if (type == 'date') {
                        if (t && _.isString(t)) {
                            t = ManagersUtils.parseDateMath(t);
                        }
                        if (f && _.isString(f)) {
                            f = ManagersUtils.parseDateMath(f);
                        }
                    }
                    if (t && f) {
                        return i18nService.format("ranges." + type, f.toString() + "|" + t.toString());
                    }
                    if (t) {
                        return i18nService.format("ranges." + type + ".leftLimit", t.toString());
                    }
                    if (f) {
                        return i18nService.format("ranges." + type + ".rightLimit", f.toString());
                    }
                };

                $scope._getSelectedIntervalAsDate = function (value) {
                    var splitted = value.split("-");
                    if ($scope.data.selectedInterval.key == "1M") {
                        return new Date(splitted[0], parseInt(splitted[1] - 1), 1, null, null, null, null)
                    } else {
                        return new Date(splitted[0], parseInt(splitted[1] - 1), splitted[2], null, null, null, null)
                    }
                };

                $scope._addConditionToQuery = function (query, condition) {
                    if (query.condition) {
                        query.condition.parameterValues.subConditions.push(condition);
                    } else {
                        query.condition = {
                            "type": "booleanCondition",
                            "parameterValues": {
                                "operator": "and",
                                "subConditions": []
                            }
                        };
                        $scope._addConditionToQuery(query, condition);
                    }
                };

                function doQuery(callback) {
                    if ($scope.data) {
                        var aggregate;
                        switch ($scope.data.selectedProperty.itemType) {
                            case "profiles":
                                aggregate = "profile.properties." + $scope.data.selectedProperty.metadata.id;
                                break;
                            case "segments":
                                aggregate = "profile.segments";
                                break;
                            default :
                                aggregate = "properties." + $scope.data.selectedProperty.metadata.id;
                        }
                        //
                        var query = {
                            aggregate: {
                                property: aggregate
                            }
                        };

                        // handle ranged properties
                        if ($scope._isPropertyRanged($scope.data.selectedProperty) && $scope.data.useRanges) {
                            var ranges;
                            if ($scope.data.selectedProperty.ipRanges.length > 0) {
                                query.aggregate.type = "ipRange";
                                query.aggregate.ipRanges = angular.copy($scope.data.selectedProperty.ipRanges);
                                ranges = query.aggregate.ipRanges;
                            } else if ($scope.data.selectedProperty.dateRanges.length > 0) {
                                query.aggregate.type = "dateRange";
                                query.aggregate.dateRanges = angular.copy($scope.data.selectedProperty.dateRanges);
                                ranges = query.aggregate.dateRanges;
                            } else if ($scope.data.selectedProperty.numericRanges.length > 0) {
                                query.aggregate.type = "numericRange";
                                query.aggregate.numericRanges = angular.copy($scope.data.selectedProperty.numericRanges);
                                ranges = query.aggregate.numericRanges;
                            }
                            $scope.ranges[$scope.data.selectedProperty.metadata.id] = {};
                            angular.forEach(ranges, function (v, k) {
                                $scope.ranges[$scope.data.selectedProperty.metadata.id][v.key] = v;
                                v.key = $scope._getI18nForRange($scope.data.selectedProperty.type, v.to, v.from);
                                $scope.ranges[$scope.data.selectedProperty.metadata.id][v.key] = v;
                            });
                        } else if ($scope.data.selectedProperty.type == "date") {
                            query.aggregate.type = "date";
                            query.aggregate.parameters = {
                                "interval": $scope.data.selectedInterval.key,
                                "format": "yyyy-MM-dd"
                            }
                        }

                        // handle fromDate and toDate
                        if ($scope.data.fromDate && $scope.data.toDate) {
                            var queryFromDate = moment($scope.data.fromDate).toISOString();
                            var queryToDate = moment($scope.data.toDate).endOf($scope.data.timelineInterval.unit).toISOString();
                            $scope._addConditionToQuery(query, $scope._getPropertyCondition(true, "timeStamp", "between", "date", true, [queryFromDate, queryToDate]));
                        }

                        // handle facets
                        if ($scope.facets) {
                            for (var key in $scope.facets) {
                                if ($scope.facets.hasOwnProperty(key)) {
                                    var facet = $scope.facets[key];
                                    if (facet.conditions && facet.conditions.length > 0) {
                                        for (var i = 0; i < facet.conditions.length; i++) {
                                            $scope._addConditionToQuery(query, facet.conditions[i]);
                                        }
                                    }
                                }
                            }
                        }


                        if ($scope.goal) {
                            goalService.getGoalReportAggregated($scope.goal.metadata.id, query).success(function (data) {
                                if ($scope.queryHook) {
                                    $scope.$parent[$scope.queryHook](data, $scope.data);
                                }

                                angular.forEach(data.split, function (v, k) {
                                    if (v.key == "_missing") {
                                        v.missing = true;
                                        v.key = i18nService.message("serverSettings.manageGoals.report.table.missing");
                                    } else if ($scope.data.selectedProperty.itemType == "segments") {
                                        if ($scope.segmentsIndexedById.hasOwnProperty(v.key)) {
                                            v.displayableKey = $scope.segmentsIndexedById[v.key].name;
                                        } else {
                                            v.displayableKey = i18nService.message("serverSettings.manageGoals.report.table.unknownSegment")
                                        }
                                    } else if (query.aggregate.type == 'date') {
                                        v.key = moment(v.key).format($scope.data.selectedInterval.aggregateFormat);
                                    }
                                    if ($scope.queryRowHook) {
                                        $scope.$parent[$scope.queryRowHook](v, $scope.data);
                                    }
                                });
                                if (callback) {
                                    callback(data.split);
                                }
                            });
                        } else {
                            var aggregateSessionQuery = {
                                "aggregate": {
                                    "type": "string",
                                    "property": aggregate
                                }
                            };

                            var tabSplit = [];
                            var queryContext = ManagersContext.proxyServletUrl + ManagersUtils.cxsContext;
                            queryService.query("session", aggregate, aggregateSessionQuery).success(function (aggregateSessionsData) {
                                if (callback) {
                                    var tabLine = {};
                                    if (aggregateSessionsData._all) {
                                        angular.forEach(aggregateSessionsData, function (v, k) {
                                            if (k != "_all") {
                                                tabLine = {};
                                                tabLine.key = k;
                                                tabLine.targetCount = v;
                                                tabLine.percentage = v / aggregateSessionsData._all;
                                                tabSplit.push(tabLine);
                                            }
                                        });
                                    }
                                    callback(tabSplit);
                                }
                            });


                        }

                    }
                }

                function loadTableColumns() {
                    var table = [];
                    if ($scope.isQualitativeReport()) {
                        table.push(DTColumnBuilder.newColumn('key').withTitle($scope.data.selectedProperty.metadata.name).renderWith(function (data, type, full) {
                            return full.displayableKey ? full.displayableKey : data;
                        }));
                        table.push(DTColumnBuilder.newColumn('startCount').withTitle(i18nService.message("serverSettings.manageGoals.views")));
                        table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message("serverSettings.manageGoals.success")));
                        table.push(DTColumnBuilder.newColumn('conversionRate').withTitle(i18nService.message("serverSettings.manageGoals.conversion")).renderWith(function (data, type, full) {
                            return ManagersUtils.getPercentage(data) + " %";
                        }));
                    } else {
                        table.push(DTColumnBuilder.newColumn('key').withTitle($scope.data.selectedProperty.metadata.name).renderWith(function (data, type, full) {
                            return full.displayableKey ? full.displayableKey : data;
                        }));
                        if ($scope.goal) {
                            table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message("serverSettings.manageGoals.success")));
                            table.push(DTColumnBuilder.newColumn('percentage').withTitle(i18nService.message("serverSettings.manageGoals.percentage")).renderWith(function (data, type, full) {
                                return ManagersUtils.getPercentage(data) + " %";
                            }));
                        } else {
                            table.push(DTColumnBuilder.newColumn('targetCount').withTitle(i18nService.message("serverSettings.manageGoals.count")));
                            table.push(
                                DTColumnBuilder
                                    .newColumn(null)
                                    .withTitle(i18nService.message("serverSettings.manageWem.listAction"))
                                    .notSortable()
                                    .renderWith($scope.actionsHtml)
                                    .withOption("width", "25%")
                                    .withOption("contentPadding", "0"));

                        }
                    }
                    if ($scope.tableColumnsHook) {
                        $scope.$parent[$scope.tableColumnsHook](table, DTColumnBuilder, $scope.data);
                    }
                    $scope.dtColumns = angular.copy(table);
                }

                $scope.dtOptions = DTOptionsBuilder.newOptions()
                    .withOption('aaSorting', [])
                    //.withOption('drawCallback', function() {
                    //    if ($scope.dtInstance) {
                    //        ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope);
                    //    }
                    //})
                    .withOption('sDom', 't')
                    .withOption('rowCallback', function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                        $('td', nRow).unbind();
                        $('td', nRow).bind('click', function () {
                            $scope.$apply(function () {
                                $scope.addFacet($scope.data.selectedProperty, aData);
                            });
                        });
                        $('td .btn.btn-icon.command-profiles', nRow).unbind();
                        $('td .btn.btn-icon.command-profiles', nRow).bind('click', function (event) {
                            event.stopPropagation();
                            $scope.openManagement();

                        });
                        return nRow;
                    });

                // watch data to refresh table
                $scope.$watch('data', function(newVal, oldVal) {
                    if ($scope.data) {
                        doQuery(function(split) {
                            //Handle geonames properties in goal report
                            if ($scope.data.selectedProperty.itemId && $scope.data.selectedProperty.itemId.indexOf('sessionAdminSubDiv') != -1) {
                                //Browsing on each table line
                                _.forEach(split, function(object, index) {
                                    //If not localhost
                                    if (object.key != 'GE' && object.key != 2500 && object.key!= "Not provided" ) {
                                        var geoId = object.key;
                                        //Request the geonames entity for the property value
                                        geonamesService.getCapital(geoId).success(function(data) {
                                            var filtered = _.filter(data, function(o) {
                                                return o.itemId == geoId;
                                            });
                                            if (filtered.length > 0 ) {
                                                //If result in geonames display the geonames entity name
                                                split[index].displayableKey = filtered[0].name
                                            }
                                            if (index == (split.length - 1)) {
                                                //Refresh datatable after geonames call for the last line
                                                $scope.split = split;
                                                $scope.dtOptions.data = split;
                                                loadTableColumns();
                                            }
                                        });
                                    }
                                });
                            }

                            //Refresh datatables with initial values
                            $scope.split = split;
                            $scope.dtOptions.data = split;
                            loadTableColumns();
                        });
                    }
                }, true);


                if ($scope.isQualitativeReport()) {
                    segmentService.getAll().success(function (segments) {
                        $scope.segmentsIndexedById = _.indexBy(segments, 'id');
                    });
                }

            }] // ,
            // templateUrl: ManagersContext.localTemplatesPath + "/goals/goal-report-table.html"
        };
    }])

    .controller('ManageGoalsListCtrl', function ($scope, $route, $location, $modal, goalService, queryService, definitionService, i18nService, currentCampaign, filteringService, profileService, _) {
        $scope.aggregateSessions = null;
        $scope.aggregateNewSessions = {};
        $scope.lineChartData = {
            labels: [],
            datasets: [],
            series: []
        };
        $scope.goalsByTag = {};
        $scope.currentCampaign = currentCampaign;
        $scope.primaryGoal = {};

        // timeline data
        $scope.timelineData = {
            toDate: moment(_.isUndefined(currentCampaign) ? moment().format("YYYY-MM") + "-01" : moment(currentCampaign.endDate).format("YYYY-MM-DD")).toDate(),
            fromDate: moment(_.isUndefined(currentCampaign) ? moment().subtract(12, "months").format("YYYY-MM") + "-01" : moment(currentCampaign.startDate).format("YYYY-MM-DD")).toDate(),
            interval: ManagersUtils.supportedTimelineIntervalesObj[_.isUndefined(currentCampaign) ? "1M" : "1d"],
            title: _.isUndefined(currentCampaign) ? "serverSettings.manageGoals.sessionsTimeline" : currentCampaign.metadata.name,
            currentCampaign: currentCampaign,
            series: [i18nService.message('serverSettings.manageGoals.allSessions'), i18nService.message('serverSettings.manageGoals.newSessions')],
            loadData: function (success) {
                var aggregateSessionQuery = {
                    "aggregate": {
                        "type": "date",
                        "parameters": {
                            "interval": this.interval.key,
                            "format": "yyyy-MM-dd"
                        }
                    },
                    "condition": {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": "scope",
                                        "comparisonOperator": "equals",
                                        "propertyValue": ManagersContext.dmfScope
                                    }
                                }
                            ]
                        }
                    }
                };
                // get all sessions
                queryService.query("session", "timeStamp", aggregateSessionQuery).success(function (aggregateSessionsData) {
                    var datasets = [];
                    datasets[0] = aggregateSessionsData;

                    // get new sessions:
                    aggregateSessionQuery.condition.parameterValues.subConditions.push({
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "profile.properties.nbOfVisits",
                            "comparisonOperator": "equals",
                            "propertyValueInteger": 1
                        }
                    });
                    queryService.query("session", "timeStamp", aggregateSessionQuery).success(function (aggregateNewSessionsData) {
                        datasets[1] = aggregateNewSessionsData;
                        success(datasets);
                    });
                });
            }
        };

        processGoals = function (goals, currentCampaign) {
            goals = filteringService.filterItems(goals, goalService);

            if (goals && goals.length > 0) {
                goals.forEach(function (goal) {
                    if (currentCampaign && goal.id == currentCampaign.primaryGoal) {
                        $scope.primaryGoal = goal;
                        goal.isPrimary = true;
                    } else {
                        if (goal.tags && goal.tags.length > 0 && !goal.hidden) {
                            var tag = $scope.goalsByTag[goal.tags[0]];
                            if (tag) {
                                if (tag.goals) {
                                    tag.goals.push(goal);
                                } else {
                                    tag.goals = [goal];
                                }
                            }
                        }
                    }
                });
            }
        };

        $scope.goalsByTag = {};
        // load goals
        // process tags and create a dictionary by tag id
        if (_.isUndefined(currentCampaign)) {
            // request goals
            $scope.goalsByTag = ManagersUtils.buildTags(ManagersUtils.goalTags, i18nService);
            goalService.getAllForSite().success(function (goals) {
                processGoals(goals);
            });
        } else {
            // request goals
            $scope.goalsByTag = ManagersUtils.buildTags(ManagersUtils.campaingTags, i18nService);
            goalService.getAllForCampaign(currentCampaign.metadata.id).success(function (goals) {
                processGoals(goals, currentCampaign);
                $scope.subGoalsAvailable = _.reduce($scope.goalsByTag, function (memo, tag) {
                        return memo + tag.goals.length
                    }, 0) > 0;
            });
        }

        $scope.refresh = function () {
            $route.reload();
        };

        $scope.subGoalsAvailable = false;

        // ABO(+) :
        $scope.quantitativeProperties = [];
        profileService.getPropertyTypes().success(function (data) {
            if (data) {
                angular.forEach(data, function (v, k) {
                    if (v && v.length > 0) {
                        angular.forEach(v, function (p) {
                            p.itemType = k;
                            p.itemTypeLabel = i18nService.message("serverSettings.manageGoals.report.table.properties." + k);
                            if (p.metadata.id == "gender") {

                                $scope.quantitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": '',
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": ''
                                };

                                $scope.$watch('timelineData', function (newVal, oldVal) {
                                    $scope.quantitativeReportData.fromDate = newVal.fromDate;
                                    $scope.quantitativeReportData.toDate = newVal.toDate;
                                    $scope.quantitativeReportData.timelineInterval = newVal.interval;
                                }, true);
                            }
                            $scope.quantitativeProperties.push(p);
                        });
                    }
                });
            }
        });
    })

    .controller('ManageGoalReportCtrl', function ($scope, $routeParams, goalService, goal, i18nService, definitionService, profileService, segmentService, $location, genericModal) {
        $scope.goalId = $routeParams.goalId;
        $scope.goal = goal;
        $scope.goalReportIntervales = ManagersUtils.supportedTimelineIntervales;

        // timeline data
        $scope.timelineData = {
            toDate: moment(moment().format("YYYY-MM"), "YYYY-MM").toDate(),
            fromDate: moment(moment().subtract(12, "months").format("YYYY-MM"), "YYYY-MM").toDate(),
            interval: $scope.goalReportIntervales[0],
            title: "serverSettings.manageGoals.report.timeline",
            goalId: $scope.goalId,
            goal: $scope.goal,
            series: [i18nService.message('serverSettings.manageGoals.report.startCountTimeline'), i18nService.message('serverSettings.manageGoals.report.targetCountTimeline')],
            loadData: function (success) {
                var query = {
                    aggregate: {
                        type: "date",
                        property: "timeStamp",
                        parameters: {
                            interval: $scope.timelineData.interval.key,
                            format: "yyyy-MM-dd"
                        }
                    }
                };
                goalService.getGoalReportAggregated($scope.timelineData.goalId, query).success(function (data) {
                    $scope.report = data;
                    $scope.report.globalStats.conversion = ManagersUtils.getPercentage($scope.report.globalStats.conversionRate);

                    var datasets = [{}, {}];

                    if (data.split && data.split.length > 0) {
                        angular.forEach(data.split, function (v, k) {
                            if (v.key) {
                                datasets[0][v.key] = v.startCount;
                                datasets[1][v.key] = v.targetCount;
                            }
                        });
                    }
                    success(datasets);
                });
            }
        };

        // get properties filter
        $scope.quantitativeProperties = [];
        $scope.qualitativeProperties = [];
        $scope.qualitativeProperties.push(
            {
                "type": 'string',
                "itemType": 'segments',
                "metadata": {
                    "id": 'segments',
                    "name": i18nService.message("serverSettings.manageGoals.report.table.properties.segments")
                },
                "itemTypeLabel": i18nService.message("serverSettings.manageGoals.report.table.properties.segments"),
            });
        profileService.getPropertyTypes().success(function (data) {
            if (data) {
                angular.forEach(data, function (v, k) {
                    if (v && v.length > 0) {
                        angular.forEach(v, function (p) {
                            p.itemType = k;
                            p.itemTypeLabel = i18nService.message("serverSettings.manageGoals.report.table.properties." + k);
                            if (p.metadata.id == "gender") {
                                $scope.qualitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": $scope.goalReportIntervales[0].key,
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": $scope.goalReportIntervales[0]
                                };
                                $scope.quantitativeReportData = {
                                    "selectedProperty": p,
                                    "fromDate": $scope.timelineData.fromDate,
                                    "toDate": $scope.timelineData.toDate,
                                    "timelineInterval": $scope.goalReportIntervales[0].key,
                                    "useRanges": true,
                                    "intervals": $scope.goalReportIntervales,
                                    "selectedInterval": $scope.goalReportIntervales[0]
                                };

                                $scope.$watch('timelineData', function (newVal, oldVal) {
                                    $scope.qualitativeReportData.fromDate = newVal.fromDate;
                                    $scope.qualitativeReportData.toDate = newVal.toDate;
                                    $scope.qualitativeReportData.timelineInterval = newVal.interval;
                                    $scope.quantitativeReportData.fromDate = newVal.fromDate;
                                    $scope.quantitativeReportData.toDate = newVal.toDate;
                                    $scope.quantitativeReportData.timelineInterval = newVal.interval;
                                }, true);
                            }
                            $scope.quantitativeProperties.push(p);
                            $scope.qualitativeProperties.push(p);
                        });
                    }
                });
            }
        });

        $scope.deleteGoal = function (id) {
            genericModal.newAlertModal({
                title: i18nService.message("serverSettings.manageWem.delete"),
                bodyMessage: i18nService.message("serverSettings.manageGoals.delete.confirm")
            }, function () {
                goalService.deleteGoal(id).success(function (data) {
                    $location.path('/goals');
                });
            });
        };
    })

    .controller('CreateGoalCtrl', function ($scope, $location, _, i18nService, goalService, queryService, notificationService, listService, ruleService, $route) {
        var tags = _.map(ManagersUtils.buildTags(ManagersUtils.goalTags, i18nService), function (tag, key) {
            return tag;
        });

        listService.query().success(function (data) {
            $scope.userLists = data.list;
        });

        $scope.goalTags = angular.copy(tags);

        var id = ManagersUtils.generateId();
        $scope.goal = {
            metadata: {
                id: id,
                name: '',
                description: '',
                enabled: false,
                scope: ManagersContext.dmfScope
            },
            type: $scope.goalTags[0]
        };

        // this callback object will be inherited in the specific edit goal ctrl and complete by this one with a save() method
        // this will allow specific edit ctrl to do specific logic to build the goal
        $scope.callback = {};

        $scope.getEditTemplate = function () {
            if ($scope.goal.type) {
                $scope.template = ManagersContext.localTemplatesPath + "/goals/goal-edit-" + $scope.goal.type.id + ".html";
            }

            return $scope.template
        };

        $scope.saveGoal = function (createForm) {
            // validate form
            if(createForm.$invalid){
                var parsedErrors = ManagersUtils.parseErrorsInForm(createForm);
                if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            // check that we don't already have a goal of this type and name
            var goalMetadata = $scope.goal.metadata;
            var type = $scope.goal.type;

            queryService.createWithCallbackIfDoesNotExist('goal', ManagersContext.dmfScope, goalMetadata.name, type.id,
                "serverSettings.manageGoals.modal.newGoal.alreadyExisting", type.name + "|" + goalMetadata.name, function () {
                    // perform save from specific callback (as defined in specific goal type controllers) to add what is needed for that particular goal type
                    $scope.callback.save(function(){
                        goalMetadata.tags = [type.id];
                        delete $scope.goal.type;
                        var lists = $scope.goal.lists;
                        delete $scope.goal.lists;

                        goalService.setGoal($scope.goal).success(function (goal) {
                            ruleService.setRule(goalService.localToServerGoalRule($scope.goal, lists));
                            $location.path("/goals");
                        });
                    }, true);
                });
        };

        $scope.resetGoal = function () {
            if($route) {
                $route.reload();
            }
        };
    })

    .controller('ManageGoalCtrl', function ($scope, $location, $route, $routeParams, goalService, goal, i18nService, notificationService) {
        $scope.goalId = $routeParams.goalId;
        $scope.goal = goal;

        $scope.getEditTemplate = function () {
            if ($scope.goal && $scope.goal.metadata && $scope.goal.metadata.tags && $scope.goal.metadata.tags.length > 0) {
                return ManagersContext.localTemplatesPath + "/goals/goal-edit-" + $scope.goal.metadata.tags[0] + ".html";
            }
        };

        $scope.save = function (editForm) {
            if(editForm.$invalid){
                var parsedErrors = ManagersUtils.parseErrorsInForm(editForm);
                if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }
            //do save
            goalService.setGoal($scope.goal).success(function () {
                if (goal.campaignId) {
                    $location.path("/campaigndashboard/" + goal.campaignId);
                } else {
                    $location.path("/goals");
                }
            });
        }
    })

    .controller('EditPageVisitGoalCtrl', function ($scope, jcrService, genericModal, i18nService) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.targetPath = ManagersUtils.getProprety($scope.goal, "targetEvent.parameterValues.pagePath");

        var doSave = function(callback) {
            // construct page visit event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "parameterValues": {
                },
                "type": "sessionCreatedEventCondition"
            };
            $scope.goal.targetEvent = {
                "parameterValues": {
                    "pagePath": $scope._goal.targetPath
                },
                "type": "pageViewEventCondition"
            };

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback, async) {
                $scope.validateGoalPaths([$scope._goal.targetPath], function(errorPaths){
                    if (callback) {
                        callback(errorPaths);
                    }
                }, async, true);
            };

            $scope.callback.save = function (callback, validate) {
                if(validate) {
                    $scope.validateGoalPaths([$scope._goal.targetPath], function(){
                        doSave(callback);
                    }, true, false);
                } else {
                    doSave(callback);
                }
            }
        }
    })

    .controller('EditFunnelGoalCtrl', function ($scope) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.targetPath = ManagersUtils.getProprety($scope.goal, "targetEvent.parameterValues.pagePath");
        $scope._goal.startPath = ManagersUtils.getProprety($scope.goal, "startEvent.parameterValues.pagePath");

        var doSave = function (callback) {
            // construct funnel events:
            $scope.goal.metadata.enabled = true;
            $scope.goal.targetEvent = {
                "parameterValues": {
                    "pagePath": $scope._goal.targetPath
                },
                "type": "pageViewEventCondition"
            };
            $scope.goal.startEvent = angular.copy($scope.goal.targetEvent);
            $scope.goal.startEvent.parameterValues.pagePath = $scope._goal.startPath;

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback, async) {
                $scope.validateGoalPaths([$scope._goal.targetPath, $scope._goal.startPath], function(errorPaths){
                    if (callback) {
                        callback(errorPaths);
                    }
                }, async, true);
            };

            $scope.callback.save = function (callback, validate) {
                if(validate) {
                    $scope.validateGoalPaths([$scope._goal.targetPath, $scope._goal.startPath], function(){
                        doSave(callback);
                    }, true, false);
                } else {
                    doSave(callback);
                }
            }
        }
    })

    .controller('EditDownloadGoalCtrl', function ($scope, jcrService, genericModal, i18nService) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.targetFile = ManagersUtils.getProprety($scope.goal, "targetEvent.parameterValues.filePath");


        var doSave = function(callback) {
            // construct download events:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "parameterValues": {
                },
                "type": "sessionCreatedEventCondition"
            };
            $scope.goal.targetEvent = {
                "parameterValues": {
                    "filePath": $scope._goal.targetFile
                },
                "type": "downloadEventCondition"
            };

            jcrService.addMixin("live", null, $scope._goal.targetFile, "wemmix:trackDownloadedFile", {}, true).success(function (data) {
                // mixin added, save goal on Apache Unomi
            });

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback, async) {
                $scope.validateGoalPaths([$scope._goal.targetFile], function(errorPaths){
                    if (callback) {
                        callback(errorPaths);
                    }
                }, async, true);
            };

            $scope.callback.save = function (callback, validate) {
                if(validate) {
                    $scope.validateGoalPaths([$scope._goal.targetFile], function(){
                        doSave(callback);
                    }, true, false);
                } else {
                    doSave(callback);
                }
            }
        }
    })

    .controller('EditLandingPageGoalCtrl', function ($scope) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        var conditions = ManagersUtils.getProprety($scope.goal, "startEvent.parameterValues.subConditions");
        if (conditions && conditions.length > 0) {
            conditions.forEach(function (condition) {
                if (condition.type == "pageViewEventCondition") {
                    $scope._goal.startPath = ManagersUtils.getProprety(condition, "parameterValues.pagePath")
                }
            })
        }

        if ($scope.goal.targetEvent && $scope.goal.targetEvent.type == "pageViewEventCondition") {
            $scope._goal.landingType = "pageView";
            $scope._goal.targetPath = ManagersUtils.getProprety($scope.goal, "targetEvent.parameterValues.pagePath");
        } else if ($scope.goal.targetEvent && $scope.goal.targetEvent.type == "booleanCondition") {
            var subConditions = ManagersUtils.getProprety($scope.goal, "targetEvent.parameterValues.subConditions");
            if (subConditions) {
                for (var i = 0; i < subConditions.length; i++) {
                    var subCondition = subConditions[i];
                    if (subCondition.type == "formEventCondition") {
                        $scope._goal.targetForm = ManagersUtils.getProprety(subCondition, "parameterValues.formId");
                        $scope._goal.landingType = "formSubmit";
                    }
                }
            }
        }

        var doSave = function(callback) {
            // construct landing page event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "parameterValues": {
                                "pagePath": $scope._goal.startPath
                            },
                            "type": "pageViewEventCondition"
                        },
                        {
                            "type": "sessionPropertyCondition",
                            "parameterValues": {
                                "propertyName": "duration",
                                "comparisonOperator": "equals",
                                "propertyValueInteger": 0
                            }
                        }
                    ]
                }
            };
            switch ($scope._goal.landingType) {
                case "pageView":
                    $scope.goal.targetEvent = {
                        "parameterValues": {
                            "pagePath": $scope._goal.targetPath
                        },
                        "type": "pageViewEventCondition"
                    };
                    break;
                case "formSubmit":
                    $scope.goal.targetEvent = {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "parameterValues": {
                                        "formId": $scope._goal.targetForm
                                    },
                                    "type": "formEventCondition"
                                },
                                {
                                    "parameterValues": {
                                        "path": $scope._goal.startPath
                                    },
                                    "type": "sourceEventPropertyCondition"
                                }
                            ]
                        }
                    };
                    break;
            }

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback, async) {
                $scope.validateGoalPaths($scope._goal.landingType == "pageView" ? [$scope._goal.startPath, $scope._goal.targetPath] : [$scope._goal.startPath], function(errorPaths){
                    if (callback) {
                        callback(errorPaths);
                    }
                }, async, true);
            };

            $scope.callback.save = function (callback, validate) {
                if(validate) {
                    $scope.validateGoalPaths($scope._goal.landingType == "pageView" ? [$scope._goal.startPath, $scope._goal.targetPath] : [$scope._goal.startPath], function(){
                        doSave(callback);
                    }, true, false);
                } else {
                    doSave(callback);
                }

            }
        }
    })

    .controller('EditVideoGoalCtrl', function ($scope) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        $scope._goal.startPath = ManagersUtils.getProprety($scope.goal, "startEvent.parameterValues.pagePath");
        var subConditions = ManagersUtils.getProprety($scope.goal, "targetEvent.parameterValues.subConditions");
        if (subConditions) {
            for (var i = 0; i < subConditions.length; i++) {
                var subCondition = subConditions[i];
                if (subCondition.type == "videoViewEventCondition") {
                    $scope._goal.targetVideo = ManagersUtils.getProprety(subCondition, "parameterValues.videoId");
                }
            }
        }

        var doSave = function (callback) {
            // construct page visit event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "parameterValues": {
                    "pagePath": $scope._goal.startPath
                },
                "type": "pageViewEventCondition"
            };
            $scope.goal.targetEvent = {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "parameterValues": {
                                "videoId": $scope._goal.targetVideo
                            },
                            "type": "videoViewEventCondition"
                        },
                        {
                            "parameterValues": {
                                "path": $scope._goal.startPath
                            },
                            "type": "sourceEventPropertyCondition"
                        }
                    ]
                }
            };

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback, async) {
                $scope.validateGoalPaths([$scope._goal.startPath], function(errorPaths){
                    if (callback) {
                        callback(errorPaths);
                    }
                }, async, true);
            };

            $scope.callback.save = function (callback, validate) {
                if(validate) {
                    $scope.validateGoalPaths([$scope._goal.startPath], function(){
                        doSave(callback);
                    }, true, false);
                } else {
                    doSave(callback);
                }

            }
        }
    })

    .controller('EditFormGoalCtrl', ['$scope', 'validationUtilsService', 'genericModal', 'i18nService',
        function ($scope, validationUtilsService, genericModal, i18nService) {
        $scope._goal = {};
        $scope.goal = $scope.$parent.goal;
        var subConditions = ManagersUtils.getProprety($scope.goal, "targetEvent.parameterValues.subConditions");
        if (subConditions) {
            for (var i = 0; i < subConditions.length; i++) {
                var subCondition = subConditions[i];
                if (subCondition.type == "formEventCondition") {
                    $scope._goal.targetForm = ManagersUtils.getProprety(subCondition, "parameterValues.formId");
                }
                if (subCondition.type == "sourceEventPropertyCondition") {
                    $scope._goal.startPath = ManagersUtils.getProprety(subCondition, "parameterValues.path");
                }
            }
        }

        var doSave = function(callback) {
            // construct page visit event:
            $scope.goal.metadata.enabled = true;
            $scope.goal.startEvent = {
                "parameterValues": {
                    "pagePath": $scope._goal.startPath
                },
                "type": "pageViewEventCondition"
            };
            $scope.goal.targetEvent = {
                "type": "booleanCondition",
                "parameterValues": {
                    "operator": "and",
                    "subConditions": [
                        {
                            "parameterValues": {
                                "formId": $scope._goal.targetForm
                            },
                            "type": "formEventCondition"
                        },
                        {
                            "parameterValues": {
                                "path": $scope._goal.startPath
                            },
                            "type": "sourceEventPropertyCondition"
                        }
                    ]
                }
            };

            if(callback) {
                callback();
            }
        };

        if ($scope.callback) {
            $scope.callback.validate = function (callback, async) {
                $scope.validateGoalPaths([$scope._goal.startPath], function(errorPaths){
                    if (callback) {
                        callback(errorPaths);
                    }
                }, async, true);
            };

            $scope.callback.save = function (callback, validate) {
                if(validate) {
                    $scope.validateGoalPaths([$scope._goal.startPath], function(){
                        validationUtilsService.getFormInPage($scope, $scope._goal.targetForm, $scope._goal.startPath, function ($form) {
                            if ($form.length > 0) {
                                doSave(callback);
                            } else {
                                genericModal.newAlertModal({
                                    title: i18nService.message("label.warning"),
                                    bodyMessage: i18nService.format("serverSettings.manageGoals.validationFormAndOk", $scope._goal.targetForm)
                                }, function () {
                                    doSave(callback);
                                });
                            }
                        });
                    }, true, false);
                } else {
                    doSave(callback);
                }

            }
        }
    }]).controller('sessionModalController',
    ['$scope', '$modalInstance', 'scopeParent',
        function ($scope, $modalInstance, scopeParent) { //Controller de la fenêtre. Il doit prend en paramètre tous les élèments du "resolve".
            // get back the query without the aggregate
            var queryWithoutAggregate = {};
            $scope._addConditionToQuery = function (query, condition) {
                if (query.condition) {
                    query.condition.parameterValues.subConditions.push(condition);
                } else {
                    query.condition = {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": []
                        }
                    };
                    $scope._addConditionToQuery(query, condition);
                }
            };

            var parentFacets = angular.copy(scopeParent.facets);
            var conditions = [];
            var clickedCondition = {
                "type": "sessionPropertyCondition",
                "parameterValues": {
                    "propertyName": scope.getCurrentProperty(),
                    "comparisonOperator": "equals",
                    "propertyValue": "male"
                }
            };
            conditions.push(clickedCondition);
            angular.forEach(parentFacets, function (value, key) {
                var facet = value;
                if (facet) {
                    if (facet.conditions && facet.conditions.length > 0) {
                        for (var i = 0; i < facet.conditions.length; i++) {
                            conditions.push(angular.copy(facet.conditions[i]));
                        }
                    }
                }
            });

            //
            scope.getCurrentProperty();

            // Ajout de la condition sur la ligne selectionnée
            // Requete au service Unomi.
            // $query('sessions');


            // recuperation de toutes les conditions.
            $scope.dtColumnsSession = angular.copy(scopeParent.dtColumns);
            $scope.dtOptionsSessions = angular.copy(scopeParent.dtOptions);


        }
    ])
    .directive('reportTable', function () {
        return {
            restrict: 'E',
            // templateUrl: ManagersContext.localTemplatesPath + "/sessions/report-table.html",
            template: 'Name: {{customer.name}} Address: {{customer.address}}',
            scope: {
                properties: '=', // stocke le type de tableau. A Supprimer.
                data: '=ngModel',
                type: "@",
                queryRowHook: "@",
                queryHook: "@",
                tableColumnsHook: "@"
            },
            controller: 'reportTableCtrl',
            resolve: null
        };

    }).controller('reportTableCtrl', ['$scope', "i18nService", "DTOptionsBuilder", "DTColumnBuilder", "$modal", function ($scope, i18nService, DTOptionsBuilder, DTColumnBuilder, $modal) {

        $scope.facets = {};
        $scope.split = [];
        $scope.ranges = {};

        $scope.setDtInstance = function (val) {
            $scope.dtInstance = val;
        };

        $scope.isNotFacetedProp = function (value, index) {
            return !$scope.facets[value.id];
        };

        $scope.showSegment = function (value, index) {
            return value.itemType != 'segments' || (value.itemType == 'segments' && $scope.isQualitativeReport());
        };

        $scope.showRangeCheckbox = function () {
            return $scope.data && $scope.data.selectedProperty && $scope._isPropertyRanged($scope.data.selectedProperty);
        };

        $scope.showDateInterval = function () {
            return $scope.data && $scope.data.selectedProperty && $scope.data.selectedProperty.type == "date";
        };

        $scope.getFacetDisplay = function (facet) {
            return facet.property.metadata.name + " : " + (facet.value.displayableKey ? ((facet.value.displayableKey.length > 50)?facet.value.displayableKey.substring(0, 50) + '...':facet.value.displayableKey) : facet.value.key);
        };


    }])
    .controller('SearchProfilesModalCtrl', ['$scope', '$modalInstance', 'profileService', 'i18nService', 'notificationService',
        function ($scope, $modalInstance, profileService, i18nService, notificationService) {
            $scope.searchCondition = angular.copy(profileService.selectedTab.searchCondition);
            $scope.conditionTypes = profileService.selectedTab.conditionTypes;
            $scope.eventConditionTypes = profileService.selectedTab.eventConditionTypes;

            $scope.searchProfiles = function () {
                if ($scope.searchProfilesForm.$invalid) {
                    $scope.searchProfilesForm.showErrors = true;

                    var parsedErrors = ManagersUtils.parseErrorsInForm($scope.searchProfilesForm);
                    if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                        ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                        return;
                    }
                }


                profileService.selectedTab.searchCondition.type = $scope.searchCondition.type;
                profileService.selectedTab.searchCondition.parameterValues = $scope.searchCondition.parameterValues;
                $modalInstance.close();
            };

            $scope.hasError = function (propertyName) {
                return ManagersUtils.hasErrors($scope.searchProfilesForm, $scope.searchProfilesForm, propertyName);
            };

            $scope.cancel = function () {
                $modalInstance.dismiss();
            };
        }])
;
