angular.module('manageProfiles', ['ui.bootstrap', 'ngResource', 'property', 'wemServices', 'wemDirectives', 'i18n', 'underscore', 'datatables', 'datatables.bootstrap', 'condition', 'wemModals', 'picker', 'chart.js', 'timeline', 'angular-md5', 'dtPager'])
    .run(['$rootScope', '$templateCache', 'i18nService', '$route',
        function ($rootScope, $templateCache, i18nService, $route) {
            $rootScope.getCurrentModuleResource = function (resourcePath) {
                return ManagersUtils.getCurrentModuleResource(resourcePath);
            };

            $rootScope.getI18n = function (key) {
                return i18nService.message(key);
            };

            $rootScope.reload = function () {
                $route.reload();
            };
            $rootScope.selectedColumns = {};
            $rootScope.selectedColumns.visitors = ['lastVisit'];
            $rootScope.selectedColumns.contacts = ['email', 'phoneNumber', 'twitterId'];
            $rootScope.selectedColumns.leads = ['firstName', 'lastName', 'leadAssignedTo'];
            $rootScope.selectedColumns.personas = ['firstName', 'lastName', 'description'];
            $rootScope.selectedColumns.defaultValues = ['email', 'lastVisit'];

            if (typeof(Storage) !== "undefined") {
                if (localStorage.selectedColumns) {
                   $rootScope.selectedColumns = JSON.parse(localStorage.selectedColumns);
                }
            }
        }])

    .controller('ManageProfileSummaryCtrl', ['$scope', '$http', '$resource', 'segmentService', 'profileService', 'queryService', '$modal',
        function ($scope, $http, $resource, segmentService, profileService, queryService, $modal) {
            if (!profileService.selectedTab) {
                profileService.selectedTab = {
                    type: 'visitors'
                };
            }
            $scope.selectedTab = profileService.selectedTab.type;

            $scope.exportProfiles = function () {
                var stringifyQuery = JSON.stringify(angular.copy(profileService.constructQueryOnCurrentSelectedTab(null, 0, -1, null)));
                profileService.exportUsingForm(stringifyQuery, profileService.selectedTab.type, "GET");
            };

            $scope.someData = [];
            $scope.visitorCount = -1;
            $scope.contactCount = -1;
            $scope.leadCount = -1;

            profileService.getCount().success(function (data) {
                $scope.visitorsCount = parseInt(data);
            });

            segmentService.getSegmentCount('contacts').success(function (data) {
                $scope.contactsCount = parseInt(data);
            });

            segmentService.getSegmentCount('leads').success(function (data) {
                $scope.leadsCount = parseInt(data);
            });

            segmentService.getSegment('contacts').success(function (data) {
                $scope.contactsSegment = data;
            });

            segmentService.getSegment('leads').success(function (data) {
                $scope.leadsSegment = data;
            });

            function getQuery(propertyName, operator, value) {
                return {
                    "aggregate": {
                        "type": "date",
                        "parameters": {
                            "interval": "1d",
                            "format": "yyyy-MM-dd"
                        }
                    },
                    "condition": {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": "and",
                            "subConditions": [
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": "scope",
                                        "comparisonOperator": "equals",
                                        "propertyValue": ManagersContext.dmfScope
                                    }
                                },
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": propertyName,
                                        "comparisonOperator": operator,
                                        "propertyValueInteger": value
                                    }
                                },
                                {
                                    "type": "sessionPropertyCondition",
                                    "parameterValues": {
                                        "propertyName": "timeStamp",
                                        "comparisonOperator": "between",
                                        "propertyValues": ["now-60d", "now"]
                                    }
                                }
                            ]
                        }
                    }
                };
            }

            function parse(data) {
                var subtract = moment().subtract(30, 'days');
                var res = [0, 0];
                angular.forEach(data, function (value, key) {
                    if (!(key.indexOf('_') == 0)) {
                        if (moment(key).isAfter(subtract)) {
                            res[1] += value
                        } else {
                            res[0] += value
                        }
                    }
                });
                if (res[0] == 0) {
                    res[2] = 0;
                    res[3] = "-"
                } else {
                    res[2] = (res[1] * 100 / res[0]) - 100;
                    res[3] = Math.abs(res[2]).toFixed(2);
                }
                return res;
            }

            // get all sessions
            queryService.query("session", "timeStamp", getQuery("profile.properties.nbOfVisits", "greaterThan", 1)).success(function (aggregateSessionsData) {
                $scope.returningVisitors = parse(aggregateSessionsData);
                // get new sessions:
            });
            queryService.query("session", "timeStamp", getQuery("profile.properties.nbOfVisits", "equals", 1)).success(function (aggregateNewSessionsData) {
                $scope.newVisitors = parse(aggregateNewSessionsData)
            });
            queryService.query("event", "timeStamp", getQuery("eventType", "equals", "view")).success(function (aggregateNewSessionsData) {
                $scope.pageViews = parse(aggregateNewSessionsData)
            });

            $scope.openSearchModal = function () {
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/search-profiles-modal.html',
                    controller: 'SearchProfilesModalCtrl',
                    size: 'vlg'
                });

                modalInstance.result.finally(function () {
                    if (profileService.selectedTab && profileService.selectedTab.dtInstance) {
                        profileService.selectedTab.dtInstance.dataTable.fnDraw()
                    }
                });
            };
        }])

    .controller('ManageProfilesCtrl', ['$rootScope', '$scope', '$resource', '$location', '$routeParams', '$http', '$q', 'i18nService', 'DTOptionsBuilder', 'DTColumnBuilder', '$modal', 'definitionService', '$compile', 'profileService', 'genericModal', 'listService',
        function ($rootScope, $scope, $resource, $location, $routeParams, $http, $q, i18nService, DTOptionsBuilder, DTColumnBuilder, $modal, definitionService, $compile, profileService, genericModal, listService) {

            profileService.selectedTab = profileService.selectedTab || {};
            profileService.selectedTab.searchCondition = profileService.selectedTab.searchCondition || {};
            profileService.selectedTab.type = $scope.profileType;
            profileService.selectedTab.deletedProfileId = $routeParams.deletedProfileId;

            $scope.fnSelectedColumnsInTab = function (column) {
                if ($rootScope.selectedColumns[profileService.selectedTab.type] == undefined) {
                    $rootScope.selectedColumns[profileService.selectedTab.type] = angular.copy($rootScope.selectedColumns.defaultValues);
                }
                if (!column.bVisible) {
                    for (var i = 0; i < $rootScope.selectedColumns[profileService.selectedTab.type].length; i++) {
                        var j = $rootScope.selectedColumns[profileService.selectedTab.type].indexOf(column.mData);
                        if (j != -1) {
                            $rootScope.selectedColumns[profileService.selectedTab.type].splice(j, 1);
                        }
                    }
                } else {
                    $rootScope.selectedColumns[profileService.selectedTab.type].push(column.mData);
                }
                if (typeof(Storage) !== "undefined") {
                    localStorage.selectedColumns = JSON.stringify($rootScope.selectedColumns);
                }
            };

            // LOAD condition and tags for condition builder for advanced search
            if (!profileService.selectedTab.conditionTypes) {
                var tags = [];
                definitionService.getTag("profileTags").success(function (tagsData) {
                    angular.forEach(tagsData.subTags, function (v) {
                        tags[v.id] = v
                    });

                    definitionService.getConditionTypeByTag("profileCondition").success(function (data) {
                        angular.forEach(data, function (condition) {
                            if (ManagersContext.conditionsTags && ManagersContext.conditionsTags[condition.id]) {
                                condition.profileTag = ManagersContext.conditionsTags[condition.id];
                            } else {
                                angular.forEach(condition.tags, function (tag) {
                                    if (tags[tag]) {
                                        condition.profileTag = tag;
                                    }
                                });
                            }
                        });

                        profileService.selectedTab.conditionTypes = data;
                    });

                    definitionService.getConditionTypeByTag("usableInPastEventCondition").success(function (data) {
                        angular.forEach(data, function (condition) {
                            if (ManagersContext.conditionsTags && ManagersContext.conditionsTags[condition.id]) {
                                condition.profileTag = ManagersContext.conditionsTags[condition.id];
                            } else {
                                angular.forEach(condition.tags, function (tag) {
                                    if (tags[tag]) {
                                        condition.profileTag = tag;
                                    }
                                });
                            }
                        });

                        profileService.selectedTab.eventConditionTypes = data;
                    });
                });
            }

            delete profileService.selectedTab.baseCondition;
            if (profileService.selectedTab.type == "contacts") {
                profileService.selectedTab.baseCondition = {
                    parameterValues: {
                        matchType: "all",
                        segments: ["contacts"]
                    },
                    type: "profileSegmentCondition"
                };
            } else if (profileService.selectedTab.type == 'leads') {
                profileService.selectedTab.baseCondition = {
                    parameterValues: {
                        matchType: "all",
                        segments: ["leads"]
                    },
                    type: "profileSegmentCondition"
                };
            } else if ($routeParams.segmentId) {
                profileService.selectedTab.baseCondition = {
                    parameterValues: {
                        matchType: "all",
                        segments: [$routeParams.segmentId]
                    },
                    type: "profileSegmentCondition"
                };
            } else if ($routeParams.campaignId) {
                var conditionPropertyName;
                if ($routeParams.goalId) {
                    conditionPropertyName = "systemProperties.goals." + $routeParams.goalId + "TargetReached";
                } else {
                    conditionPropertyName = "systemProperties.campaigns." + $routeParams.campaignId + "Engaged";
                }
                profileService.selectedTab.baseCondition = {
                    parameterValues: {
                        propertyName: conditionPropertyName,
                        comparisonOperator: "exists"
                    },
                    type: "profilePropertyCondition"
                };
            } else if ($routeParams.listId) {
                profileService.selectedTab.baseCondition = {
                    parameterValues: {
                        propertyName: "systemProperties.lists",
                        comparisonOperator: "equals",
                        propertyValue: $routeParams.listId
                    },
                    type: "profilePropertyCondition"
                };
            } else if ($routeParams.scoringId) {
                profileService.selectedTab.baseCondition =
                {
                    "parameterValues": {
                        "subConditions": [
                            {
                                parameterValues: {
                                    propertyName: "scores." + $routeParams.scoringId,
                                    comparisonOperator: "exists"
                                },
                                type: "profilePropertyCondition"
                            },
                            {
                                parameterValues: {
                                    propertyName: "scores." + $routeParams.scoringId,
                                    "comparisonOperator": "greaterThan",
                                    "propertyValueInteger": 0
                                },
                                type: "profilePropertyCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                };
            }

            var existingProfilePropertyTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/existingProperties?tagId=profileProperties&itemType=' + ($scope.profileType == 'personas' ? "persona" : "profile"), {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });

            $scope.loading = true;

            function serverData(sSource, aoData, fnCallback, oSettings) {
                var aoDataCopy = angular.copy(aoData);
                // shift for remove the draw object, it's just a counter increment each time the table is draw
                aoDataCopy.shift();
                if ($scope.aoData && angular.equals($scope.aoData, aoDataCopy) && $scope.previousResult) {
                    fnCallback($scope.previousResult);
                    ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                }
                $scope.aoData = aoDataCopy;
                var draw = _.find(aoData, function (x) {
                    return x.name == "draw"
                }).value;

                var start = _.find(aoData, function (x) {
                    return x.name == "start"
                }).value;

                var length = _.find(aoData, function (x) {
                    return x.name == "length"
                }).value;

                var order = _.map(_.find(aoData, function (x) {
                    return x.name == "order"
                }).value, function (o) {
                    var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                    if (prop == "id") {
                        prop = "itemId";
                    } else if (prop == "scores" && $routeParams.scoringId) {
                        prop = "scores." + $routeParams.scoringId;
                    } else {
                        prop = "properties." + prop;
                    }
                    return prop + ":" + o.dir
                }).join(",");

                var search = _.find(aoData, function (x) {
                    return x.name == "search"
                }).value.value;
                if (search != '') {
                    search = '*' + search.trim() + '*';
                }else {
                    search = "***";
                }

                profileService.search(profileService.constructQueryOnCurrentSelectedTab(search, start, length, order), profileService.selectedTab.type).success(function (data) {
                    $scope.profiles = {};

                    console.log("manage-profiles.js: " + data.list.length + " profiles loaded out of a total " + data.totalSize + " profiles.");
                    // copy the profile information to a format that can be used by the table component we use
                    var profileTable = [];
                    for (var i = 0; i < data.list.length; i++) {
                        var profileTableRow = {};
                        var currentProfile = data.list[i];
                        $scope.profiles[currentProfile.itemId] = currentProfile;
                        profileTableRow.id = currentProfile.itemId;
                        for (var j = 0; j < $scope.existingPropertyTypes.length; j++) {
                            var existingPropertyType = $scope.existingPropertyTypes[j];
                            var propertyId = existingPropertyType.metadata.id;
                            if (currentProfile.properties.hasOwnProperty(propertyId)) {
                                if (existingPropertyType.type == "date") {
                                    profileTableRow[propertyId] = new Date(currentProfile.properties[propertyId]);
                                } else {
                                    profileTableRow[propertyId] = currentProfile.properties[propertyId];
                                }
                            } else {
                                profileTableRow[propertyId] = "";
                            }
                        }
                        if ($routeParams.scoringId) {
                            profileTableRow['scores'] = currentProfile.scores[$routeParams.scoringId];
                        }
                        profileTable.push(profileTableRow);
                    }

                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": data.totalSize,
                        "iTotalDisplayRecords": data.totalSize,
                        "aaData": profileTable
                    };
                    $scope.searchError = false;
                    $scope.previousResult = results;
                    fnCallback(results);
                    $scope.loading = false;
                    ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                }).error(function (data, status, headers, config) {
                    $scope.searchError = true;
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                });
            }

            $scope.dtOptions = DTOptionsBuilder.newOptions()
                .withOption('pageLength', 25)
                .withOption('processing', true)
                .withOption('serverSide', true)
                .withOption('sDom', 't')
                .withOption('createdRow', createdRow)
                .withFnServerData(serverData);

            function createdRow(row, data, dataIndex) {
                console.log("Row is created: " + row);
                // Recompiling so we can bind Angular directive to the DT
                $compile(angular.element(row).contents())($scope);
            }

            $scope.existingPropertyTypes = existingProfilePropertyTypesResource.get({}, function (propertyTypes) {
                console.log("manage-profiles.js: " + propertyTypes.length + " propertyTypes loaded.");

                $scope.dtColumns = [DTColumnBuilder.newColumn('id').withTitle('ID').renderWith(function (data, type, properties, meta) {
                    return ManagersUtils.getProfileDisplayName(i18nService, properties);
                })];
                if (profileService.selectedTab.type == 'contacts') {
                    $scope.dtOptions.withOption("order", [[0, 'asc']]);
                }

                var selectedColumns = $rootScope.selectedColumns.defaultValues;
                if ($rootScope.selectedColumns[profileService.selectedTab.type] != undefined) {
                    selectedColumns = $rootScope.selectedColumns[profileService.selectedTab.type]
                }

                for (var i = 0; i < propertyTypes.length; i++) {
                    var propertyType = propertyTypes[i];
                    var propTypeId = propertyType.metadata.id;
                    var columnTitle = propertyType.metadata.name;
                    var col = DTColumnBuilder.newColumn(propTypeId).withTitle(propertyType.metadata.name);
                    col.bVisible = true;
                    col.excludedFromSelection = false;

                    if (propTypeId == "lastVisit") {
                        $scope.dtOptions.withOption("order", [[i + 1, 'desc']]);
                        col.renderWith(function (data, type, full, meta) {
                            return moment(data).fromNow();
                        })
                    }

                    if (!( _.contains(selectedColumns, propTypeId))) {
                        col.notVisible();
                    }

                    if (profileService.selectedTab.type == 'visitors') {
                        if (propTypeId == "lastVisit") {
                            $scope.dtOptions.withOption("order", [[i + 1, 'desc']]);
                            col.renderWith(function (data, type, full, meta) {
                                return moment(data).fromNow();
                            })
                        }
                    } else if (profileService.selectedTab.type == 'leads') {
                        if (propTypeId == "leadAssignedTo") {
                            $scope.dtOptions.withOption("order", [[i + 1, 'asc']]);
                        }
                    } else if (profileService.selectedTab.type == 'personas') {
                        if (propTypeId == "lastName") {
                            $scope.dtOptions.withOption("order", [[i + 1, 'asc']]);
                        }
                    }
                    $scope.dtColumns.push(col);
                }

                if ($routeParams.scoringId) {
                    $scope.dtColumns.push(DTColumnBuilder.newColumn('scores').withTitle(i18nService.message('serverSettings.manageProfiles.score')));
                    $scope.dtOptions.withOption("order", [[$scope.dtColumns.length - 1, 'desc']]);
                }

                $scope.dtColumns.push(DTColumnBuilder.newColumn(null)
                    .withTitle(i18nService.message('serverSettings.manageWem.listAction'))
                    .notSortable()
                    .renderWith(actionsHtml));
                $scope.loading = false;
            });

            $scope.isAdvanceSearchActive = function () {
                return profileService.selectedTab && profileService.selectedTab.searchCondition && profileService.selectedTab.searchCondition.type;
            };

            $scope.deleteProfile = function (profileId) {
                genericModal.newAlertModal({
                    title: i18nService.message("serverSettings.manageWem.delete"),
                    bodyMessage: i18nService.message("serverSettings.manageProfiles.delete.confirm")
                }, function () {
                    profileService.deleteProfile(profileId, profileService.selectedTab.type).success(function (data) {
                        profileService.selectedTab.deletedProfileId = profileId;
                        $scope.updateDatatable();
                    });
                });
            };

            function actionsHtml(data, type, full, meta) {
                return "<button title=\"" + i18nService.message("serverSettings.manageProfiles.gotoProfile") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + full.id + "')\"><span class=\"md md-edit\"></span></button> " +
                    (($scope.profileType == "personas") ? "" : "<button title=\"" + i18nService.message("serverSettings.manageLists") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"openListsModal('" + full.id + "')\" ><span class=\"md md-list\"></span></button>") +
                    "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteProfile('" + full.id + "')\"><span class=\"md md-delete\"></span></button>";
            }

            $scope.goToEdit = function (id) {
                if ($routeParams.segmentId) {
                    $location.path("/segment/" + $routeParams.segmentId + "/profile/" + id);
                } else if ($routeParams.campaignId) {
                    var path = "/campaign/" + $routeParams.campaignId;
                    if ($routeParams.goalId) {
                        path += "/converted/" + $routeParams.goalId;
                    }
                    path += "/profile/" + id;
                    $location.path(path);
                } else if ($routeParams.listId) {
                    $location.path("/list/" + $routeParams.listId + "/profile/" + id);
                } else if ($routeParams.scoringId) {
                    $location.path("/scoring/" + $routeParams.scoringId + "/profile/" + id);
                } else if ($scope.profileType == 'personas') {
                    $location.path("/persona/" + id);
                } else {
                    $location.path("/profile/" + id);
                }
            };
            
            $scope.updateSearch = function(){
                if ($scope.dtInstance) {
                    $scope.dtInstance.DataTable.search($scope.searchField).draw();
                }
                $scope.emptySearchField = ($scope.searchField == '')
            };

            $scope.$watch("pageLength", function (newValue, oldValue) {
                if (newValue && parseInt(newValue) != oldValue && oldValue && $scope.dtInstance) {
                    $scope.dtInstance.DataTable.page.len(newValue).draw();
                }
            });

            $scope.updateDatatable = function () {
                $scope.dtInstance.dataTable.fnDraw();
            };

            $scope.openListsModal = function (profileId) {
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/add-to-lists-modal.html',
                    controller: 'AddToListsModalCtrl',
                    size: 'sm',
                    resolve: {
                        "lists": function ($q) {
                            var lists = $q.defer();
                            listService.query().success(function (data) {
                                lists.resolve(data.list);
                            });
                            return lists.promise;
                        },
                        "selectedProfile": function () {
                            return $scope.profiles[profileId];
                        }
                    }
                });
                modalInstance.result.then(function (profile) {
                    // refresh table in case we are in the list members and list have been removed for the given profile
                    if ($routeParams.listId && $.inArray($routeParams.listId, profile.systemProperties.lists) == -1) {
                        profileService.selectedTab.deletedProfileId = profileId;
                        $scope.updateDatatable();
                    }
                })
            };

            $scope.openSearchModal = function () {
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/search-profiles-modal.html',
                    controller: 'SearchProfilesModalCtrl',
                    size: 'vlg'
                });

                modalInstance.result.finally(function () {
                    if (profileService.selectedTab && profileService.selectedTab.dtInstance) {
                        profileService.selectedTab.dtInstance.dataTable.fnDraw()
                    }
                });
            };

            $scope.exportProfiles = function () {
                var stringifyQuery = JSON.stringify(angular.copy(profileService.constructQueryOnCurrentSelectedTab(null, 0, -1, null)));
                profileService.exportUsingForm(stringifyQuery, profileService.selectedTab.type, "GET");
            };

            $scope.openSaveAsModal = function (type) {
                $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/save-as-' + type + '-modal.html',
                    controller: 'SaveAsModalCtrl',
                    size: 'lg',
                    resolve: {
                        "type": function () {
                            return type;
                        }
                    }
                });
            };

            $scope.cancelSearch = function () {
                profileService.selectedTab.searchCondition = {};
                $scope.updateDatatable();
            };

            $scope.setDtInstance = function (instance) {
                $scope.dtInstance = instance;
                profileService.selectedTab.dtInstance = instance;
            };
        }])

    .directive("manageProfilesTable", function () {
        return {
            restrict: "E",
            scope: {
                profileType: '@'
            },
            templateUrl: function (elem, attrs) {
                if (attrs.profileType == "visitors") {
                    return ManagersContext.localTemplatesPath + "/profiles/visitors-table.html";
                } else if (attrs.profileType == "personas") {
                    return ManagersContext.localTemplatesPath + "/profiles/personas-table.html";
                } else {
                    return ManagersContext.localTemplatesPath + "/profiles/contacts-table.html";
                }

            },
            controller: "ManageProfilesCtrl"
        }
    })

    .controller('AddToListsModalCtrl', ['$scope', '$modalInstance', 'profileService', 'i18nService', 'lists', 'selectedProfile',
        function ($scope, $modalInstance, profileService, i18nService, lists, selectedProfile) {
            $scope.lists = lists;
            $scope.selectedProfile = selectedProfile;

            if (!$scope.selectedProfile.systemProperties.lists) {
                $scope.selectedProfile.systemProperties.lists = [];
            }

            $scope.save = function () {
                profileService.saveProfile($scope.selectedProfile, "profiles").success(function (data) {
                    $modalInstance.close($scope.selectedProfile);
                });
            };

            $scope.cancel = function () {
                $modalInstance.dismiss('cancel');
            };
        }])

    .controller('AddScoreModifiersModalCtrl', ['$scope', '$modalInstance', 'profileService', 'i18nService', 'scorings', 'selectedProfile',
        function ($scope, $modalInstance, profileService, i18nService, scorings, selectedProfile) {
            $scope.scorings = scorings;
            $scope.selectedProfile = selectedProfile;

            if (!$scope.selectedProfile.systemProperties.scoreModifiers) {
                $scope.selectedProfile.systemProperties.scoreModifiers = {};
            }

            $scope.save = function () {
                _.each($scope.selectedProfile.systemProperties.scoreModifiers, function (value, key) {
                    if (!value) {
                        delete $scope.selectedProfile.systemProperties.scoreModifiers[key];
                    }
                });

                profileService.saveProfile($scope.selectedProfile, "profiles").success(function (data) {
                    $modalInstance.close($scope.selectedProfile);
                });
            };

            $scope.cancel = function () {
                $modalInstance.dismiss('cancel');
            };
        }])

    .controller('AddToSegmentsModalCtrl', ['$scope', '$modalInstance', 'profileService', 'i18nService', 'segments', 'selectedProfile',
        function ($scope, $modalInstance, profileService, i18nService, segments, selectedProfile) {
            $scope.segments = segments;
            $scope.selectedProfile = selectedProfile;

            if (!$scope.selectedProfile.segments) {
                $scope.selectedProfile.segments = [];
            }

            $scope.save = function () {
                profileService.saveProfile($scope.selectedProfile, "personas").success(function (data) {
                    $modalInstance.close($scope.selectedProfile);
                });
            };

            $scope.cancel = function () {
                $modalInstance.dismiss('cancel');
            };
        }])

    .controller('SearchProfilesModalCtrl', ['$scope', '$modalInstance', 'profileService', 'i18nService', 'notificationService',
        function ($scope, $modalInstance, profileService, i18nService, notificationService) {
            $scope.searchCondition = angular.copy(profileService.selectedTab.searchCondition);
            $scope.conditionTypes = profileService.selectedTab.conditionTypes;
            $scope.eventConditionTypes = profileService.selectedTab.eventConditionTypes;

            $scope.searchProfiles = function () {
                if ($scope.searchProfilesForm.$invalid) {
                    $scope.searchProfilesForm.showErrors = true;

                    var parsedErrors = ManagersUtils.parseErrorsInForm($scope.searchProfilesForm);
                    if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                        ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                        return;
                    }
                }


                profileService.selectedTab.searchCondition.type = $scope.searchCondition.type;
                profileService.selectedTab.searchCondition.parameterValues = $scope.searchCondition.parameterValues;
                $modalInstance.close();
            };

            $scope.hasError = function (propertyName) {
                return ManagersUtils.hasErrors($scope.searchProfilesForm, $scope.searchProfilesForm, propertyName);
            };

            $scope.cancel = function () {
                $modalInstance.dismiss();
            };
        }])

    .controller('AddPropertyModalCtrl', ['$scope', '$modalInstance', '$resource', 'profileService', 'i18nService', 'notificationService', 'propertiesByTag', 'tagId',
        function ($scope, $modalInstance, $resource, profileService, i18nService, notificationService, propertiesByTag, tagId) {
            $scope.type = "string";
            $scope.valueTypes = [];
            var valueTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/definitions/values', {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            valueTypesResource.get(function (valueTypes) {
                console.log("manage-profile.js: Loaded " + valueTypes.length + " value types from Apache Unomi.");
                for (var i = 0; i < valueTypes.length; i++) {
                    $scope.valueTypes.push(valueTypes[i].id);
                }
            });

            $scope.cancel = function () {
                $modalInstance.dismiss('cancel');
            };

            $scope.saveProperty = function () {
                $scope.addProfilePropertyForm.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm($scope.addProfilePropertyForm);

                if (parsedErrors && Object.keys(parsedErrors).length > 0) {
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }

                var tagProperties = propertiesByTag[tagId];
                var propertyType = {
                    "defaultValue": "",
                    "itemId": $scope.id,
                    "itemType": "propertyType",
                    "metadata": {
                        "id": $scope.id,
                        "name": $scope.name
                    },
                    "rank": tagProperties[tagProperties.length - 1].rank + 1,
                    "tags": [tagId],
                    "target": "profiles",
                    "type": $scope.type,
                    "multivalued": $scope.isMultivalued
                };
                profileService.createPropertyType(propertyType).success(function (result) {
                    if (result) {
                        tagProperties.push(propertyType);
                        notificationService.success(i18nService.format("serverSettings.manageProfiles.property.add.success", $scope.id));
                        $modalInstance.close();
                    } else {
                        notificationService.error(i18nService.format("serverSettings.manageProfiles.property.add.failed", $scope.id));
                    }
                });
            };

            $scope.hasError = function (propertyName) {
                return ManagersUtils.hasErrors($scope.addProfilePropertyForm, $scope.addProfilePropertyForm, propertyName);
            };
        }])

    .controller('SaveAsModalCtrl', ['$scope', '$modalInstance', 'segmentService', 'i18nService', 'listService', 'profileService', 'notificationService', 'type',
        function ($scope, $modalInstance, segmentService, i18nService, listService, profileService, notificationService, type) {
            $scope.metadata = {
                id: ManagersUtils.generateId(),
                scope: ManagersContext.dmfScope,
                enabled: true
            };

            $scope.shared = false;

            if (type == "segment") {
                $scope.save = function () {
                    var currentSegment = {
                        metadata: $scope.metadata,
                        condition: profileService.constructCondition(profileService.selectedTab.baseCondition, profileService.selectedTab.searchCondition, null, false, profileService.selectedTab.type)
                    };

                    segmentService.setSegment(currentSegment, $scope.shared)
                        .success(function () {
                            notificationService.success(i18nService.format('serverSettings.manageSegments.create.success', currentSegment.metadata.name));
                        })
                        .error(function () {
                            notificationService.error(i18nService.message('serverSettings.manageSegments.create.error'));
                        });
                    $modalInstance.close();
                };
            } else if (type == "list") {
                $scope.save = function () {
                    var list = {
                        metadata: $scope.metadata
                    };
                    listService.setList(list, $scope.shared)
                        .success(function () {
                            var update = {
                                condition: profileService.constructCondition(profileService.selectedTab.baseCondition, profileService.selectedTab.searchCondition, null, true, profileService.selectedTab.type),
                                propertyName: "systemProperties.lists",
                                propertyValue: $scope.metadata.id,
                                strategy: "addValue"
                            };
                            profileService.batchProfilesUpdate(update)
                                .success(function () {
                                    notificationService.success(i18nService.format('serverSettings.manageLists.create.success', list.metadata.name));
                                    if (profileService.selectedTab && profileService.selectedTab.dtInstance) {
                                        profileService.selectedTab.dtInstance.dataTable.fnDraw()
                                    }
                                })
                                .error(function () {
                                    notificationService.error(i18nService.message('serverSettings.manageLists.create.error'));
                                });
                        });
                    $modalInstance.close();
                };
            }

            $scope.cancel = function () {
                $modalInstance.dismiss('cancel');
            }
        }])

    .controller('ManageProfileCtrl', ['$scope', '$route', '$routeParams', '$resource', 'i18nService', 'profileService', 'profile', '$http', 'DTOptionsBuilder', 'DTColumnBuilder', 'listService', '$modal', 'segmentService', 'goalService', '$location', 'queryService', '$window', '$compile', 'genericModal', 'notificationService', 'scoringService', 'jcrService',
        function ($scope, $route, $routeParams, $resource, i18nService, profileService, profile, $http, DTOptionsBuilder, DTColumnBuilder, listService, $modal, segmentService, goalService, $location, queryService, $window, $compile, genericModal, notificationService, scoringService, jcrService) {
            if (profile.itemType == 'persona') {
                $scope.profileType = 'personas';
            }
            if (!profileService.selectedTab) {
                profileService.selectedTab = {};
                profileService.selectedTab.type = 'visitors';
            }
            $scope.countries = ManagersContext.countries;
            $scope.loaded = true;
            $scope.propertyTypesLoaded = false;
            $scope.propertiesLoaded = false;
            $scope.profileId = $routeParams.profileId;
            $scope.currentProfile = profile;
            $scope.siteKey = ManagersContext.dmfScope;
            $scope.sites = {};
            // compute age from birth date
            if (profile.properties.birthDate) {
                var birthdate = new Date(profile.properties.birthDate);
                var cur = new Date();
                var diff = cur - birthdate;
                $scope.age = i18nService.format("profilesProperty.age.value", Math.floor(diff / 31536000000).toString());
            }

            $scope.currentProfileHasSegments = function () {
                return $scope.currentProfile.segments && !angular.equals([], $scope.currentProfile.segments);
            };

            $scope.currentProfileHasLists = function () {
                return $scope.currentProfile.systemProperties.lists && !angular.equals([], $scope.currentProfile.systemProperties.lists);
            };

            $scope.currentProfileHasScores = function () {
                return $scope.currentProfile.scores && !angular.equals({}, $scope.currentProfile.scores);
            };

            function updateProfileView() {
                $scope.profileName = ManagersUtils.getProfileDisplayName(i18nService, $scope.currentProfile.properties);

                if (profile.itemType == 'persona') {
                    $scope.profileRB = 'managePersonas';
                } else if ($scope.currentProfile.segments.indexOf("leads") != -1) {
                    $scope.profileRB = "manageLeads";
                } else if ($scope.currentProfile.segments.indexOf("contacts") != -1) {
                    $scope.profileRB = "manageContacts";
                } else {
                    $scope.profileRB = "manageVisitors";
                }
                $scope.profileTypeLabel = i18nService.message("serverSettings." + $scope.profileRB + ".profileType");
            }

            updateProfileView();

            if ($scope.currentProfile.properties.firstVisit) {
                $scope.firstVisitTimeAgo = moment($scope.currentProfile.properties.firstVisit).fromNow();
            }
            if ($scope.currentProfile.properties.lastVisit) {
                $scope.lastVisitTimeAgo = moment($scope.currentProfile.properties.lastVisit).fromNow();
            }

            var propertyTypeRootTagResource = $resource(ManagersContext.proxyServletUrl + '/cxs/definitions/tags/profileProperties?filterHidden=true', {}, {
                'get': {
                    method: 'GET',
                    isArray: false
                }
            });
            var propertyTypesByTagResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/properties/tags/:tagId', {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var existingSessionPropertyTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/existingProperties?tagId=sessionProperties&itemType=session', {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });

            $scope.propertiesByTag = {};
            propertyTypeRootTagResource.get(function (rootTag) {
                console.log("manage-profile.js: Loaded " + rootTag.subTags.length + " property type tags from Apache Unomi.");
                for (var i = 0; i < rootTag.subTags.length; i++) {
                    var propertyTag = rootTag.subTags[i];
                    $scope.propertiesByTag[propertyTag.id] = propertyTypesByTagResource.get({tagId: propertyTag.id}, function (propertyTypes) {
                        console.log("manage-profile.js: Loaded " + propertyTypes.length + " property types from Apache Unomi.");
                    });
                }
            });

            $scope.middleIndex = function (properties) {
                if (properties) {
                    return Math.ceil(properties.length / 2);
                }
                return 0;
            };

            $scope.loading = true;
            
            $scope.saveProfile = function () {
                if ($scope.currentProfile.itemId == "newPersona") {
                    $scope.currentProfile.itemId = ManagersUtils.generateId();
                    profileService.saveProfile($scope.currentProfile, $scope.profileType).success(function (data) {
                        notificationService.success(i18nService.message("serverSettings.manageProfiles.save.success"));
                        $location.path('/persona/' + $scope.currentProfile.itemId);
                    });
                } else {
                    profileService.saveProfile($scope.currentProfile, $scope.profileType).success(function (savedProfile) {
                        $scope.currentProfile = savedProfile;
                        updateProfileView();
                        notificationService.success(i18nService.message("serverSettings.manageProfiles.save.success"));
                    });
                }
            };

            $scope.getPropertyInputType = function (property) {
                if (property.type == "integer") {
                    return "number";
                } else if (property.type == "email") {
                    return "email";
                } else if (property.type == "date") {
                    return "date";
                }
                return "text";
            };

            $scope.resetProfile = function () {
                profileService.getProfile($scope.profileId, $scope.profileType).success(function (profile) {
                    console.log("manage-profile.js: Loaded profile " + profile.itemId + " succesfully from Apache Unomi.");
                    $scope.currentProfile = profile;
                    $scope.propertiesLoaded = true;
                    $scope.loaded = $scope.propertiesLoaded && $scope.propertyTypesLoaded;
                });
            };

            $scope.deleteProfile = function (profileId) {
                genericModal.newAlertModal({
                    title: i18nService.message("serverSettings." + $scope.profileRB + ".delete"),
                    bodyMessage: i18nService.message("serverSettings." + $scope.profileRB + ".delete.confirm")
                }, function () {
                    profileService.deleteProfile($scope.profileId, $scope.profileType).success(function (data) {
                        $location.url("/" + (profile.itemType == 'persona' ? "personas" : "profiles") + "?deletedProfileId=" + $scope.profileId);
                    });
                });
            };

            $scope.exportProfile = function () {
                var stringifyQuery = JSON.stringify(
                    condition = {
                        "condition": {
                            "type": "profilePropertyCondition",
                            "parameterValues": {
                                "comparisonOperator": "equals",
                                "propertyName": "itemId",
                                "propertyValue": profile.itemId
                            }
                        }
                    }
                );
                profileService.exportUsingForm(stringifyQuery, profileService.selectedTab.type, "GET");

            };

            $scope.timelineColours = [
                {
                    fillColor: "rgba(220,220,220,0.2)",
                    strokeColor: "rgba(220,220,220,1)",
                    pointColor: "rgba(220,220,220,1)",
                    pointStrokeColor: "#fff",
                    pointHighlightFill: "#fff",
                    pointHighlightStroke: "rgba(220,220,220,1)"
                },
                {
                    fillColor: "rgba(132, 84, 198, 0.2)",
                    strokeColor: "#8454c6",
                    pointColor: "#8454c6",
                    pointStrokeColor: "#fff",
                    pointHighlightFill: "#fff",
                    pointHighlightStroke: "#8454c6"
                }
            ];
            $scope.timelineData = {
                toDate: new Date(),
                fromDate: moment().subtract(1, "weeks").toDate(),
                interval: ManagersUtils.supportedTimelineIntervalesObj["1d"],
                series: [i18nService.message('serverSettings.manageProfiles.pageViews'), i18nService.message('serverSettings.manageProfiles.sessions')],
                loadData: function (success) {
                    if ($scope.profileId) {
                        var aggregateSessionQuery = {
                            "aggregate": {
                                "type": "date",
                                "parameters": {
                                    "interval": this.interval.key,
                                    "format": "yyyy-MM-dd"
                                }
                            },
                            "condition": {
                                "type": "sessionPropertyCondition",
                                "parameterValues": {
                                    "propertyName": "profileId",
                                    "propertyValue": $scope.profileId,
                                    "comparisonOperator": "equals"
                                }
                            }
                        };
                        var aggregatePageViewsQuery = {
                            "aggregate": {
                                "type": "date",
                                "parameters": {
                                    "interval": this.interval.key,
                                    "format": "yyyy-MM-dd"
                                }
                            },
                            "condition": {
                                "type": "booleanCondition",
                                "parameterValues": {
                                    "operator": "and",
                                    "subConditions": [
                                        {
                                            "type": "eventTypeCondition",
                                            "parameterValues": {
                                                "eventTypeId": "view"
                                            }
                                        },
                                        {
                                            "type": "sessionPropertyCondition",
                                            "parameterValues": {
                                                "propertyName": "profileId",
                                                "propertyValue": $scope.profileId,
                                                "comparisonOperator": "equals"
                                            }
                                        }
                                    ]
                                }
                            }
                        };
                        // get all sessions
                        queryService.query("event", "timeStamp", aggregatePageViewsQuery).success(function (aggregatePageViewsData) {
                            var datasets = [];
                            datasets[0] = aggregatePageViewsData;
                            queryService.query("session", "timeStamp", aggregateSessionQuery).success(function (aggregateSessionData) {
                                datasets[1] = aggregateSessionData;
                                success(datasets);
                            });
                        });
                    }
                }
            };

            var interests = _.map($scope.currentProfile.properties.interests, function (num, key) {
                return {name: key, value: num};
            });
            interests = _.sortBy(interests, 'value');
            interests = _.last(interests, ManagersUtils.maxInterestsInChart);
            interests = _.sortBy(interests, 'name');

            $scope.interestsData = [_.pluck(interests, 'value')];
            $scope.interestsLabels = _.pluck(interests, 'name');
            $scope.interestsOptions = {
                responsive: true
            };
            $scope.interestsColors = [
                {
                    fillColor: "#F1DE30",
                    strokeColor: "rgba(51,153,255,0)",
                    pointColor: "rgba(51,153,255,0)",
                    pointHighlightFill: "#fff",
                    pointHighlightStroke: "rgba(51,153,255,0)"
                }
            ];
            if ($scope.profileId) {
                var aggregateSessionQuery = {
                    "aggregate": {
                        "type": "string"
                    },
                    "condition": {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "profileId",
                            "propertyValue": $scope.profileId,
                            "comparisonOperator": "equals"
                        }
                    }
                };
                queryService.query("session", "properties.deviceCategory", aggregateSessionQuery).success(function (aggregateDeviceData) {
                    var desktop = aggregateDeviceData["Personal computer"] ? aggregateDeviceData["Personal computer"] : 0;
                    var mobile = aggregateDeviceData["Smartphone"] ? aggregateDeviceData["Smartphone"] : 0;
                    var tablet = aggregateDeviceData["Tablet"] ? aggregateDeviceData["Tablet"] : 0;
                    var total = aggregateDeviceData._filtered;
                    $scope.desktopUsage = (desktop * 100 / total).toFixed(1) + "%";
                    $scope.mobileUsage = (mobile * 100 / total).toFixed(1) + "%";
                    $scope.tabletUsage = (tablet * 100 / total).toFixed(1) + "%";
                    $scope.othersUsage = ((total - desktop - mobile - tablet) * 100 / total).toFixed(1) + "%";
                });

                queryService.query("session", "properties.userAgentNameAndVersion", aggregateSessionQuery).success(function (aggregateAgentData) {
                    var userAgents = [];
                    for (var key in aggregateAgentData) {
                        if (key.indexOf('_') != 0) {
                            userAgents.push({name: key, value: aggregateAgentData[key]});
                        }
                    }
                    var maxUserAgent = _.max(userAgents, function (userAgent) {
                        return userAgent.value;
                    });

                    if (maxUserAgent.name) {
                        var userAgentNameAndVersion = maxUserAgent.name.split("@@");
                        $scope.highestTraficBrowser = userAgentNameAndVersion[0] + " " + userAgentNameAndVersion[1];
                        $scope.highestTraficBrowserImage = $scope.getCurrentModuleResource('/images/browsers/' + userAgentNameAndVersion[0].toLowerCase().replace(" ", "-") + '.png');
                    }
                    $scope.highestTraficBrowserUsage = (maxUserAgent.value * 100 / aggregateAgentData._filtered).toFixed(1) + "%";
                });

                queryService.query("session", "properties.countryAndCity", aggregateSessionQuery).success(function (aggregateLocationData) {
                    var locations = [];
                    for (var key in aggregateLocationData) {
                        if (key.indexOf('_') != 0) {
                            locations.push({name: key, value: aggregateLocationData[key]});
                        }
                    }
                    var maxLocation = _.max(locations, function (location) {
                        return location.value;
                    });

                    if (maxLocation.name) {
                        var countryAndCity = maxLocation.name.split("@@");
                        $scope.userLocationCountry = countryAndCity[0] == 'null' ? i18nService.message("serverSettings.manageProfiles.location.unknown") : countryAndCity[0];
                        $scope.userLocationCity = countryAndCity[1] == 'null' ? i18nService.message("serverSettings.manageProfiles.location.unknown") : countryAndCity[1];
                        $scope.userLocationState = countryAndCity[2] != 'null' ? countryAndCity[2] : '';
                        $scope.userLocationSubState = countryAndCity[3] != 'null' ? countryAndCity[3] : '';
                    }
                });
            }
            $scope.segmentsIndexedById = {};
            segmentService.getAll().success(function (segments) {
                var segmentsFiltered = _.reject(segments, function (segment) {
                   return segment.scope != 'systemscope' && segment.scope != ManagersContext.dmfScope;
                });
                $scope.segmentsIndexedById = _.indexBy(segments, 'id');
                $scope.segmentsFiltered = segmentsFiltered;
            });
            $scope.getSegmentById = function (segmentId) {
                return $scope.segmentsIndexedById[segmentId];
            };
            $scope.getScopeLabelById = function (id, type) {
                var collection = $scope.segmentsIndexedById;
                if(type === 'list'){
                    collection = $scope.profileListsIndexedById;
                }
                if(!_.isEmpty(collection)) {
                    var currentScope = collection[id].scope
                    if (currentScope != ManagersContext.dmfScope) {
                        var stringToReturn = ' (';
                        if (currentScope === 'systemscope') {
                            stringToReturn += i18nService.message("serverSettings.manageSegments.shared");
                        } else {
                            stringToReturn += currentScope;
                        }
                        stringToReturn += ')';
                        return stringToReturn
                    } else {
                        return '';
                    }
                }
            }
            $scope.goalsIndexedById = {};
            goalService.getAll().success(function (goal) {
                $scope.goalsIndexedById = _.indexBy(goal, 'id');
            });
            $scope.getGoalById = function (goalId) {
                return $scope.goalsIndexedById[goalId];
            };
            $scope.getGoalClass = function(scope){
                return ManagersContext.dmfScope == scope?"in-scope-goal":"out-scope-goal";
            }
            $scope.isProfileScoreNotified = function (scoringId) {
                return $scope.currentProfile.systemProperties.notificationAck &&
                    $scope.currentProfile.systemProperties.notificationAck.scoring &&
                    $scope.currentProfile.systemProperties.notificationAck.scoring[scoringId] &&
                    $scope.currentProfile.systemProperties.notificationAck.scoring[scoringId] > 0;
            };

            $scope.scoringsIndexedById = {};
            scoringService.getAll().success(function (scorings) {
                var scoringsFiltered = _.reject(scorings, function (scoring) {
                    return scoring.scope != 'systemscope' && scoring.scope != ManagersContext.dmfScope;
                });
                $scope.scoringsIndexedById = _.indexBy(scorings, 'id');
                $scope.scoringsFiltered = scoringsFiltered;
            });
            $scope.getScoringById = function (scoringId) {
                return $scope.scoringsIndexedById[scoringId];
            };

            $scope.openListsModal = function () {
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/add-to-lists-modal.html',
                    controller: 'AddToListsModalCtrl',
                    size: 'sm',
                    resolve: {
                        "lists": function () {
                            return $scope.userLists;
                        },
                        "selectedProfile": function () {
                            return angular.copy($scope.currentProfile);
                        }
                    }
                });
                modalInstance.result.then(function (profile) {
                    $scope.currentProfile.systemProperties.lists = profile.systemProperties.lists;
                });
            };

            $scope.openScoreModifiersModal = function () {
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/add-score-modifiers-modal.html',
                    controller: 'AddScoreModifiersModalCtrl',
                    resolve: {
                        "scorings": function () {
                            return $scope.scoringsFiltered;
                        },
                        "selectedProfile": function () {
                            return angular.copy($scope.currentProfile);
                        }
                    }
                });
                modalInstance.result.then(function (profile) {
                    $scope.resetProfile();
                });
            };

            $scope.openSegmentsModal = function () {
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/add-to-segments-modal.html',
                    controller: 'AddToSegmentsModalCtrl',
                    resolve: {
                        "segments": function () {
                            return $scope.segmentsFiltered;
                        },
                        "selectedProfile": function () {
                            return angular.copy($scope.currentProfile);
                        }
                    }
                });
                modalInstance.result.then(function (profile) {
                    $scope.currentProfile.segments = profile.segments;
                });
            };

            $scope.openAddPropertyModal = function (tagId) {
                $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/profiles/add-property-modal.html',
                    controller: 'AddPropertyModalCtrl',
                    resolve: {
                        propertiesByTag: function () {
                            return $scope.propertiesByTag;
                        },
                        "tagId": function () {
                            return tagId;
                        }
                    }
                });
            };

            $scope.confirmDeleteProperty = function (propertyId) {
                genericModal.newAlertModal({
                    title: i18nService.message("serverSettings.manageWem.delete"),
                    bodyMessage: i18nService.format("serverSettings.manageProfiles.property.delete.confirm", propertyId)
                }, function () {
                    profileService.deletePropertyType(propertyId).success(function () {
                        $scope.currentProfile.properties[propertyId] = null;
                        for (tag in $scope.propertiesByTag) {
                            if ($scope.propertiesByTag.hasOwnProperty(tag)) {
                                $scope.propertiesByTag[tag] = _.filter($scope.propertiesByTag[tag], function (prop) {
                                    return prop.itemId != propertyId;
                                });
                            }
                        }
                        notificationService.success(i18nService.format("serverSettings.manageProfiles.property.delete.success", propertyId));
                    });
                });
            };

            $scope.cardsEdition = [];
            $scope.toggleEdition = function (tag) {
                $scope.cardsEdition[tag] = $scope.cardsEdition[tag] ? !$scope.cardsEdition[tag] : true;
            };

            $scope.isEditionEnabled = function (tag) {
                return $scope.cardsEdition[tag];
            };

            // lists
            $scope.profileListsIndexedById = {};
            $scope.userLists = [];
            $scope.profileLists = [];
            $scope.getListById = function (listId) {
                return $scope.profileListsIndexedById[listId];
            };
            var queryListWithoutScope = {
                "condition": {
                    "parameterValues": {
                        "propertyName": "metadata.scope",
                        "comparisonOperator": "exists"
                    },
                    "type": "sessionPropertyCondition"
                },
                "limit": 50,
                "offset": 0,
                "sortby": null,
                "forceRefresh": false
            };

            listService.query(queryListWithoutScope).success(function (data) {
                $scope.userLists = _.reject(data.list, function (entry) {
                    return entry.scope != 'systemscope' && entry.scope != ManagersContext.dmfScope;
                });
                $scope.profileListsIndexedById = _.indexBy(data.list, 'id');
            });

            $scope.dtOptions = DTOptionsBuilder.newOptions()
                .withOption('processing', true)
                .withOption('serverSide', true)
                .withOption('sDom', 't')
                .withOption('createdRow', createdRow)
                .withOption('pageLength', 6)
                .withFnServerData(serverData)
                .withPaginationType('full_numbers')
                .withOption("order", [[0, 'desc']]);

            function createdRow(row, data, dataIndex) {
                console.log("Row is created: " + row);
                // Recompiling so we can bind Angular directive to the DT
                $compile(angular.element(row).contents())($scope);
            }

            $scope.existingSessionPropertyTypes = existingSessionPropertyTypesResource.get({}, function (sessionPropertyTypes) {
                console.log("manage-profiles.js: " + sessionPropertyTypes.length + " propertyTypes loaded.");

                $scope.dtColumns = [
                    DTColumnBuilder.newColumn('timeStamp').withTitle(i18nService.message('session.timeStamp')).renderWith(function (data, type, full, meta) {
                        return "<a href=\"\" ng-click=\"goToSessionEdit('" + full.id + "')\" class=\"userID\">" + moment(data).format('LL') + "</a>";
                    }),
                    DTColumnBuilder.newColumn('duration').withTitle(i18nService.message('session.duration')).renderWith(function (data, type, full, meta) {
                        return moment.duration(data).humanize();
                    }),
                    DTColumnBuilder.newColumn('size').withTitle(i18nService.message('session.size'))
                ];

                if (_.some(sessionPropertyTypes, function (propertyType) {
                        return propertyType.itemId == "countryName";
                    })) {
                    $scope.dtColumns.push(DTColumnBuilder.newColumn('countryName').withTitle(i18nService.message('sessionsProperty.countryName')));
                }

                $scope.loading = false;
            });

            $scope.isObject = function (property) {
                return angular.isObject(property);
            };

            function serverData(sSource, aoData, fnCallback, oSettings) {
                var draw = _.find(aoData, function (x) {
                    return x.name == "draw"
                }).value;

                var start = _.find(aoData, function (x) {
                    return x.name == "start"
                }).value;

                var length = _.find(aoData, function (x) {
                    return x.name == "length"
                }).value;

                var order = _.map(_.find(aoData, function (x) {
                    return x.name == "order"
                }).value, function (o) {
                    var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                    if (prop == "countryName") {
                        prop = "properties." + prop;
                    }
                    return prop + ":" + o.dir
                }).join(",");

                var config = {
                    url: ManagersContext.proxyServletUrl + "/cxs/profiles/" + $scope.profileId + "/sessions",
                    method: 'get',
                    params: {
                        offset: start,
                        size: length,
                        sort: order
                    }
                };

                $http(config)
                    .success(function (data, status, headers, config) {
                        console.log("manage-profiles.js: " + data.list.length + " sessions loaded out of a total " + data.totalSize + " sessions.");
                        // copy the session information to a format that can be used by the table component we use
                        var sessionTable = [];
                        for (var i = 0; i < data.list.length; i++) {
                            var sessionTableRow = {};
                            var session = data.list[i];
                            sessionTableRow.id = session.itemId;
                            sessionTableRow.timeStamp = new Date(session.timeStamp);
                            sessionTableRow.duration = session.duration;
                            sessionTableRow.size = session.size;
                            sessionTableRow.countryName = session.properties.countryName ? session.properties.countryName : '';
                            sessionTable.push(sessionTableRow);
                        }

                        var results = {
                            "sEcho": draw,
                            "iTotalRecords": data.totalSize,
                            "iTotalDisplayRecords": data.totalSize,
                            "aaData": sessionTable
                        };
                        fnCallback(results);
                        ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                    }).error(function (data, status, headers, config) {
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                });
            }

            $scope.setDtInstance = function (instance) {
                $scope.dtInstance = instance;
            };

            $scope.getTabTitle = function (key) {
                return i18nService.message(key);
            };

            $scope.getPreviousPage = function () {
                // otherwise if we have search id, route to the search
                if ($routeParams.searchId) {
                    // if we have a profile and session id in the route, route back to the profile view
                    return '#/search/' + $routeParams.searchId;
                } else if ($routeParams.segmentId) {
                    return '#/segment/' + $routeParams.segmentId + '/profiles';
                } else if ($routeParams.campaignId) {
                    var previousPage = "#/campaign/" + $routeParams.campaignId;
                    if ($routeParams.goalId) {
                        previousPage += "/converted/" + $routeParams.goalId;
                    } else {
                        previousPage += "/profiles";
                    }
                    return previousPage;
                } else if ($routeParams.scoringId) {
                    return '#/scoring/' + $routeParams.scoringId + '/profiles';
                } else if ($routeParams.listId) {
                    return '#/list/' + $routeParams.listId + '/profiles';
                } else if ($scope.profileType == 'personas') {
                    return '#/personas';
                } else if ($routeParams.queryID) {
                    return '#/incoming-search/' + $routeParams.queryID;
                } else if ($routeParams.referrerId) {
                    return '#/referrer/' + $routeParams.referrerId;
                }

                return '#/profiles';
            };

            $scope.goToSessionEdit = function (sessionId) {
                var route;
                if ($routeParams.searchId) {
                    // if we have a profile and session id in the route, route back to the profile view
                    route = '/search/' + $routeParams.searchId;
                } else if ($routeParams.segmentId) {
                    route = '/segment/' + $routeParams.segmentId;
                } else if ($routeParams.campaignId != undefined) {
                    route = '/campaign/' + $routeParams.campaignId;
                    if ($routeParams.goalId) {
                        route += "/converted/" + $routeParams.goalId;
                    }
                } else if ($routeParams.scoringId) {
                    route = '/scoring/' + $routeParams.scoringId;
                } else if ($routeParams.listId) {
                    route = '/list/' + $routeParams.listId;
                } else if ($routeParams.queryID) {
                    route = '/incoming-search/' + $routeParams.queryID;
                } else if ($routeParams.referrerId) {
                    route = '/referrer/' + $routeParams.referrerId;
                } else {
                    route = '';
                }

                $location.path(route + '/profile/' + $routeParams.profileId + '/' + sessionId);
            };
            $scope.goals = [];
            if ($scope.currentProfile.systemProperties) {
                angular.forEach($scope.currentProfile.systemProperties.goals, function (v, k) {
                    if (k.substring(k.length - "TargetReached".length) == "TargetReached") {
                        var goalId = k.substring(0, k.length - "TargetReached".length);
                        goalService.getGoal(goalId).success(function(response){
                            var currentGoal = response.metadata;
                            if($scope.sites[currentGoal.scope] === undefined){
                                jcrService.doGetOnPath("default",ManagersContext.mainResourceLocale,"/sites/"+currentGoal.scope+"/properties/j__title").success(function(jcrResponse){
                                    console.log("manage-profiles.js: Get site "+currentGoal.scope+" in jcr");
                                    $scope.sites[currentGoal.scope] = jcrResponse.value;
                                    currentGoal['sitename'] = jcrResponse.value;
                                    $scope.goals.push({
                                        id: goalId,
                                        date: moment(v),
                                        name: currentGoal.name,
                                        scope: currentGoal.scope,
                                        applyon: "( "+i18nService.message("serverSettings.manageProfiles.site")+": "+currentGoal.sitename+" )"
                                    })
                                }).error(function(){
                                    $scope.goals.push({
                                        id: goalId,
                                        date: moment(v),
                                        name: currentGoal.name,
                                        scope: currentGoal.scope,
                                        applyon: "( "+i18nService.message("serverSettings.manageProfiles.scope")+": "+currentGoal.scope+" )"
                                    });
                                });
                            } else {
                                $scope.goals.push({
                                    id: goalId,
                                    date: moment(v),
                                    name: currentGoal.name,
                                    scope: currentGoal.scope,
                                    applyon: "( "+i18nService.message("serverSettings.manageProfiles.site")+": "+$scope.sites[currentGoal.scope]+" )"
                                });
                            }

                        });
                    }
                });
            }
            $scope.goals = _.sortBy($scope.goals, function (goal) {
                return moment(goal.date);
            }).reverse();

            if ($routeParams.searchId || $routeParams.queryID || $routeParams.referrerId) {
                $scope.headerBgClass = "blue-bg";
            } else if ($routeParams.segmentId) {
                $scope.headerBgClass = "pink-bg";
            } else if ($routeParams.campaignId) {
                $scope.headerBgClass = "purple-bg";
            } else if ($routeParams.listId) {
                $scope.headerBgClass = "pink-bg";
            } else if ($routeParams.scoringId) {
                $scope.headerBgClass = "dark-yellow-bg";
            } else if ($scope.profileType == 'personas') {
                $scope.headerBgClass = "purple-bg";
            } else {
                $scope.headerBgClass = "green-bg";
            }
        }])

    .controller('ManageSessionCtrl', ['$scope', '$routeParams', '$resource', 'i18nService', 'DTOptionsBuilder', 'DTColumnBuilder', '$http',
        function ($scope, $routeParams, $resource, i18nService, DTOptionsBuilder, DTColumnBuilder, $http) {
            // Scroll to the top
            window.scrollTo(0, 0);

            $scope.sessionId = $routeParams.sessionId;
            $scope.profileId = $routeParams.profileId;

            $scope.loaded = true;
            $scope.propertyTypesLoaded = false;
            $scope.propertiesLoaded = false;

            $scope.currentSession = {};

            var sessionResourceURL = '/cxs/profiles/sessions/:sessionId';

            var sessionResource = $resource(ManagersContext.proxyServletUrl + sessionResourceURL);
            var valueTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/definitions/values', {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var propertyTypeRootTagResource = $resource(ManagersContext.proxyServletUrl + '/cxs/definitions/tags/sessionProperties', {}, {
                'get': {
                    method: 'GET',
                    isArray: false
                }
            });
            var allPropertyTypesResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/properties/sessions', {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var propertyTypesByTagResource = $resource(ManagersContext.proxyServletUrl + '/cxs/profiles/properties/tags/:tagId', {}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });

            $scope.getMomentDate = function (date) {
                return moment(date).format('LL');
            };

            $scope.getMomentDuration = function (duration) {
                return moment.duration(duration).humanize();
            };

            $scope.resetSession = function () {
                $scope.currentSession = sessionResource.get({sessionId: $scope.sessionId}, function (profile) {
                    console.log("manage-session.js: Loaded profile " + profile.itemId + " succesfully from Apache Unomi.");
                    $scope.propertiesLoaded = true;
                    $scope.loaded = $scope.propertiesLoaded && $scope.propertyTypesLoaded;

                    if ($scope.currentSession.properties.hasOwnProperty('userAgentName')) {
                        $scope.browserImage = $scope.getCurrentModuleResource('/images/browsers/' + $scope.currentSession.properties.userAgentName.toLowerCase().replace(" ", "-") + '.png');
                    }

                    if ($scope.currentSession.properties.hasOwnProperty('deviceCategory')) {
                        var deviceCategory = ($scope.currentSession.properties.deviceCategory || '').toLowerCase();
                        if (deviceCategory == "personal computer") {
                            $scope.device = "desktop";
                        } else if (deviceCategory == "smartphone") {
                            $scope.device = "mobile";
                        } else if (deviceCategory == "tablet") {
                            $scope.device = "tablet";
                        } else {
                            $scope.device = "others";
                        }
                        $scope.deviceLabel = i18nService.message("serverSettings.manageSessions.device." + $scope.device);
                    }
                });
            };

            $scope.saveSession = function () {
                $scope.currentSession.$save({sessionId: $scope.sessionId},
                    function () {
                        notificationService.success(i18nService.message("serverSettings.manageSessions.save.success"));
                    }
                );
            };

            $scope.valueTypes = valueTypesResource.get(function (valueTypes) {
                console.log("manage-session.js: Loaded " + valueTypes.length + " value types from Apache Unomi.");

                $scope.propertyTypeRootTag = propertyTypeRootTagResource.get(function (rootTag) {
                    console.log("manage-session.js: Loaded " + rootTag.subTags.length + " property type tags from Apache Unomi.");
                    var loadedNewTranslationParts = false;
                    for (var i = 0; i < rootTag.subTags.length; i++) {
                        var propertyTag = rootTag.subTags[i];
                        propertyTag.propertyTypes = propertyTypesByTagResource.get({tagId: propertyTag.id}, function (propertyTypes) {
                            console.log("manage-session.js: Loaded " + propertyTypes.length + " property types from Apache Unomi.");
                            var loadedNewTranslationParts = false;
                            for (var j = 0; j < propertyTypes.length; j++) {
                                var propertyType = propertyTypes[j];
                            }
                        })
                    }
                });

                $scope.existingPropertyTypes = allPropertyTypesResource.get(function (propertyTypes) {
                    console.log("manage-session.js: Loaded " + propertyTypes.length + " propertyTypes from Apache Unomi.");
                    $scope.propertyTypesLoaded = true;
                    $scope.loaded = $scope.propertiesLoaded && $scope.propertyTypesLoaded;
                });

            });

            function infoColumn(data, type, full, meta) {
                if (data == "null") {
                    return "";
                }
                if (full.eventType == 'view') {
                    return i18nService.message("serverSettings.manageSessions.event.viewedPage") + " : <a href=\"" + data + "\" target=\"_blank\">" + data + "</a>";
                }
                // event.eventType == 'goal'
                return i18nService.message("serverSettings.manageSessions.event.reachedGoal") + " : " + data;
            }

            $scope.dtOptions = DTOptionsBuilder.newOptions()
                .withOption('processing', true)
                .withOption('serverSide', true)
                .withOption('sDom', 't')
                .withFnServerData(serverData)
                .withPaginationType('full_numbers')
                .withOption("order", [[0, 'asc']]);

            $scope.dtColumns = [
                DTColumnBuilder.newColumn('timeStamp').withTitle(i18nService.message('event.time')).renderWith(function (data) {
                    if (data == $scope.currentSession.timeStamp) {
                        return ""
                    }
                    return i18nService.message('event.timeAfter') + " " + moment.duration(moment(data).diff(moment($scope.currentSession.timeStamp))).humanize()
                }),
                DTColumnBuilder.newColumn('info').withTitle(i18nService.message('serverSettings.manageSessions.event.info')).renderWith(infoColumn).notSortable(),
            ];

            function serverData(sSource, aoData, fnCallback, oSettings) {
                var draw = _.find(aoData, function (x) {
                    return x.name == "draw"
                }).value;

                var start = _.find(aoData, function (x) {
                    return x.name == "start"
                }).value;

                var length = _.find(aoData, function (x) {
                    return x.name == "length"
                }).value;

                var order = _.map(_.find(aoData, function (x) {
                    return x.name == "order"
                }).value, function (o) {
                    return "timeStamp:" + o.dir
                }).join(",");

                var search = _.find(aoData, function (x) {
                    return x.name == "search"
                }).value.value;
                if (search != '') {
                    search = '*' + search.trim() + '*';
                }

                var config = {
                    url: ManagersContext.proxyServletUrl + "/cxs/profiles/sessions/" + $scope.sessionId + "/events",
                    method: 'get',
                    params: {
                        eventTypes: ["view", "goal"],
                        q: search,
                        offset: start,
                        size: length,
                        sort: order
                    }
                };

                $http(config)
                    .success(function (data, status, headers, config) {
                        console.log("manage-profiles.js: " + data.list.length + " events loaded out of a total " + data.totalSize + " events.");
                        // copy the event information to a format that can be used by the table component we use
                        var eventTable = [];
                        for (var i = 0; i < data.list.length; i++) {
                            var eventTableRow = {};
                            var event = data.list[i];
                            eventTableRow.timeStamp = event.timeStamp;
                            if (event.eventType == 'view') {
                                eventTableRow.info = event.target.properties.pageInfo.destinationURL;
                            } else { // event.eventType == 'goal'
                                eventTableRow.info = event.target.metadata.name;
                            }
                            eventTableRow.eventType = event.eventType;
                            eventTable.push(eventTableRow);
                        }

                        var results = {
                            "sEcho": draw,
                            "iTotalRecords": data.totalSize,
                            "iTotalDisplayRecords": data.totalSize,
                            "aaData": eventTable
                        };
                        fnCallback(results);
                        ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                    }).error(function (data, status, headers, config) {
                    var results = {
                        "sEcho": draw,
                        "iTotalRecords": 0,
                        "iTotalDisplayRecords": 0,
                        "aaData": []
                    };
                    fnCallback(results);
                });
            }

            $scope.setDtInstance = function (instance) {
                $scope.dtInstance = instance;
            };

            $scope.getPreviousPage = function () {
                // if we have search id, route to the search
                if ($routeParams.searchId) {
                    return '#/search/' + $routeParams.searchId + '/profile/' + $routeParams.profileId;
                } else if ($routeParams.segmentId) {
                    return '#/segment/' + $routeParams.segmentId + '/profile/' + $routeParams.profileId;
                } else if ($routeParams.campaignId) {
                    var previousPage = '#/campaign/' + $routeParams.campaignId;
                    if ($routeParams.goalId) {
                        previousPage += "/converted/" + $routeParams.goalId;
                    }
                    previousPage += '/profile/' + $routeParams.profileId
                    return previousPage;
                } else if ($routeParams.listId) {
                    return '#/list/' + $routeParams.listId + '/profile/' + $routeParams.profileId;
                } else if ($routeParams.scoringId) {
                    return '#/scoring/' + $routeParams.scoringId + '/profile/' + $routeParams.profileId;
                } else if ($routeParams.queryID) {
                    return '#/incoming-search/' + $routeParams.queryID + '/profile/' + $routeParams.profileId;
                } else if ($routeParams.referrerId) {
                    return '#/referrer/' + $routeParams.referrerId + '/profile/' + $routeParams.profileId;
                }

                if ($routeParams.profileId) {
                    return '#/profile/' + $routeParams.profileId;
                }

                return '#/profiles';
            };

            if ($routeParams.searchId || $routeParams.queryID || $routeParams.referrerId) {
                $scope.headerBgClass = "blue-bg";
            } else if ($routeParams.segmentId) {
                $scope.headerBgClass = "pink-bg";
            } else if ($routeParams.campaignId) {
                $scope.headerBgClass = "purple-bg";
            } else if ($routeParams.listId) {
                $scope.headerBgClass = "pink-bg";
            } else if ($routeParams.scoringId) {
                $scope.headerBgClass = "dark-yellow-bg";
            } else if ($scope.profileType == 'personas') {
                $scope.headerBgClass = "purple-bg";
            } else {
                $scope.headerBgClass = "green-bg";
            }

            $scope.resetSession();
        }]);
