wem = {
    setCookie: function (cname, cvalue, exdays) {
        var expires = "";
        if (exdays) {
            var d = new Date();
            d.setTime(d.getTime() + (exdays * 24 * 60 * 60 * 1000));
            expires = "; expires=" + d.toUTCString();
        }
        document.cookie = cname + "=" + cvalue + expires + "; path=/";
    },

    getCookie:function (cname) {
        var name = cname + "=";
        var ca = document.cookie.split(';');
        for(var i=0; i<ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0)==' ') c = c.substring(1);
            if (c.indexOf(name) == 0) return c.substring(name.length,c.length);
        }
        return null;
    },

    removeCookie: function (cname) {
        "use strict";
        wem.setCookie(cname, "", -1);
    },

    registerEvent: function (event) {
        if (window.digitalData) {
            if (window.cxs) {
                console.error("already loaded, too late ..");
            } else {
                window.digitalData.events = window.digitalData.events || [];
                window.digitalData.events.push(event);
            }
        } else {
            window.digitalData = {};
            window.digitalData.events = window.digitalData.events || [];
            window.digitalData.events.push(event);
        }
    },

    registerFilterCallbacks: function (filter, callback) {
        if (window.digitalData) {
            if (window.cxs) {
                console.error("already loaded, too late ..");
            } else {
                console.log("wem: digitalData object present but not loaded, registering filter callback...");
                window.digitalData.filterCallback = window.digitalData.filterCallback || [];
                window.digitalData.filterCallback.push({filter: filter, callback: callback});
            }
        } else {
            window.digitalData = {};
            window.digitalData.filterCallback = window.digitalData.filterCallback || [];
            window.digitalData.filterCallback.push({filter: filter, callback: callback});
        }
    },

    getFilterTargets: function () {
        var targets = [];
        if (window.digitalData.filterCallback) {
            for (var i = 0; i < window.digitalData.filterCallback.length; i++) {
                var currentNodeFilters = window.digitalData.filterCallback[i].filter;
                for (var j = 0; j < currentNodeFilters.filters.length; j++) {
                    var currentNodeFilter = currentNodeFilters.filters[j];
                    for (var k = 0; k < currentNodeFilter.appliesOn.length; k++) {
                        var applyOnEntry = currentNodeFilter.appliesOn[k];
                        targets.push(applyOnEntry);
                    }
                }
            }
        }
        return targets;
    },

    registerCallbacks: function (onLoadCallback) {
        if (window.digitalData) {
            if (window.cxs) {
                console.log("wem: digitalData object loaded, calling on load callback immediately and registering update callback...");
                if (onLoadCallback) {
                    onLoadCallback(window.digitalData);
                }
            } else {
                console.log("wem: digitalData object present but not loaded, registering load callback...");
                if (onLoadCallback) {
                    window.digitalData.loadCallbacks = window.digitalData.loadCallbacks || [];
                    window.digitalData.loadCallbacks.push(onLoadCallback);
                }
            }
        } else {
            console.log("wem: No digital data object found, creating and registering update callback...");
            window.digitalData = {};
            if (onLoadCallback) {
                window.digitalData.loadCallbacks = [];
                window.digitalData.loadCallbacks.push(onLoadCallback);
            }
        }
    },

    resolvePersonalization: function (successfulFilters, noConditionsFilters, filters, strategy, fallback) {
        var selectedFilter;

        if (strategy == "random") {
            function randomIndex(array) {
                return Math.floor(Math.random() * array.length);
            }

            successfulFilters = successfulFilters.concat(noConditionsFilters);
            selectedFilter = successfulFilters[randomIndex(successfulFilters)];
        } else {
            function compareFn(a, b) {
                // sort in reverse order of priority so that the first element is the highest priority one
                return -(a.priority - b.priority);
            }

            function compareFnPage(a, b) {
                // Sort on position for pages (Only in preview panel)
                return (a.position - b.position);
            }

            // defaults to : (strategy == 'matching-first')
            // No-condition filters are considered as always displayed the priority/order will define
            // which one is selected
            successfulFilters = successfulFilters.concat(noConditionsFilters);
            if (successfulFilters.length > 0) {
                if (typeof pagePersonalization !== 'undefined' && pagePersonalization!=null) { // Sort on position for pages (Only in preview panel)
                    successfulFilters.sort(compareFnPage);
                } else { // Sort on priority for contents/other
                    successfulFilters.sort(compareFn);
                }
                selectedFilter = successfulFilters[0];
            }
        }

        if(!selectedFilter && fallback && filters[fallback]) {
            selectedFilter = {
                "filterId": fallback,
                "variantWrapper": filters[fallback].variantWrapper,
                "variantWrapperPath": filters[fallback].variantWrapperPath,
                "variantPath": filters[fallback].variantPath,
                "pagePath": filters[fallback].pagePath,
                "priority": filters[fallback].priority,
                "content": filters[fallback].content
            }
        }

        return selectedFilter;
    },

    registerPersonalization: function(filters, strategy, target, fallback, ajax, resultCallback) {
        var noConditionsFilters = [];
        for (var filterId in filters) {
            if (filters.hasOwnProperty(filterId)) {
                if (filters[filterId].filter.filters.length > 0) {
                    //register filter
                    wem.registerFilterCallbacks(filters[filterId].filter, function () {
                    });
                } else {
                    //no filter assume that the content pass the test
                    var element = {
                        "filterId": filterId,
                        "variantWrapper": filters[filterId].variantWrapper,
                        "variantWrapperPath": filters[filterId].variantWrapperPath,
                        "variantPath": filters[filterId].variantPath,
                        "pagePath": filters[filterId].pagePath,
                        "priority": filters[filterId].priority,
                        "content": filters[filterId].content
                    };
                    if (typeof pagePersonalization !== 'undefined' && pagePersonalization!=null) {//Page personalization use position for filters sort (Only in preview panel)
                        element.position = filters[filterId].variantPosition;
                    }
                    noConditionsFilters.push(element);
                }
            }
        }

        // look on filter results and define the content to display base on the strategy
        wem.registerCallbacks(function () {
            var successfulFilters = [];
            for (var filterId in cxs.filteringResults) {
                if (cxs.filteringResults.hasOwnProperty(filterId)) {
                    if (filters.hasOwnProperty(filterId)) {
                        if (cxs.filteringResults[filterId]) {
                            var element = {
                                "filterId": filterId,
                                "variantWrapper": filters[filterId].variantWrapper,
                                "variantWrapperPath": filters[filterId].variantWrapperPath,
                                "variantPath": filters[filterId].variantPath,
                                "pagePath": filters[filterId].pagePath,
                                "priority": filters[filterId].priority,
                                "content": filters[filterId].content
                            };
                            if (typeof pagePersonalization !== 'undefined' && pagePersonalization!=null) {//Page personalization use position for filters sort (Only in preview panel)
                                element.position = filters[filterId].variantPosition;
                            }
                            successfulFilters.push(element);
                        }
                    }
                }
            }

            var selectedFilter = wem.resolvePersonalization(successfulFilters, noConditionsFilters, filters, strategy, fallback);

            if (resultCallback != null) {
                // execute callback
                resultCallback(successfulFilters, selectedFilter);
            } else {
                if (selectedFilter) {
                    if (ajax) {
                        wem.loadContent(selectedFilter.content, target);
                    } else {
                        var targetContents = document.getElementById(target).children;
                        for (var index in targetContents) {
                            var el = targetContents.item(index);
                            if(el){
                                el.style.display = (el.id == selectedFilter.content)?'':'none';
                            }
                        }
                    }

                    //Trigger variant display event for personalization
                    var displayWemVariantEvent = document.createEvent("CustomEvent");
                    displayWemVariantEvent.initCustomEvent('displayWemVariant', false, false, {
                        type: "personalization",
                        variantWrapperPath: selectedFilter.variantWrapperPath,
                        variantPath: selectedFilter.variantPath,
                        pagePath: selectedFilter.pagePath,
                        variantWrapper: selectedFilter.variantWrapper,
                        variant: selectedFilter.filterId
                    });
                    document.dispatchEvent(displayWemVariantEvent);
                } else {
                    if (ajax) {
                        document.getElementById(target).innerHTML = '';
                    } else {
                        var elements = document.getElementById(target).children;
                        for (var index in elements) {
                            var el = elements.item(index);
                            el.style.display = 'none';
                        }
                    }
                }
            }
        });
    },

    anonymizeProfile: function (successCallback, errorCallback) {
        wem.ajax({
            url: wem.proxyServletUrl + '/cxs/privacy/profiles/' + cxs.profileId + '/anonymize',
            type: 'POST',
            async: true,
            contentType: "application/x-www-form-urlencoded",
            dataType: 'application/json',
            data: '',
            success: function() {
                successCallback();
            },
            error: function (xhr) {
                errorCallback(xhr);
            }
        });
    },

    togglePrivateBrowsing: function (successCallback, errorCallback) {
        if (cxs.anonymousBrowsing) {
            wem.ajax({
                url: wem.proxyServletUrl + '/cxs/privacy/profiles/' + cxs.profileId + '/anonymousBrowsing',
                type: 'DELETE',
                async: true,
                contentType: "application/x-www-form-urlencoded",
                dataType: 'application/json',
                success: function() {
                    successCallback();
                },
                error: function (xhr) {
                    errorCallback(xhr);
                }
            });
        } else {
            wem.ajax({
                url: wem.proxyServletUrl + '/cxs/privacy/profiles/' + cxs.profileId + '/anonymousBrowsing?anonymizePastBrowsing=true',
                type: 'POST',
                async: true,
                contentType: "application/x-www-form-urlencoded",
                dataType: 'application/json',
                data: '',
                success: function() {
                    successCallback();
                },
                error: function (xhr) {
                    errorCallback(xhr);
                }
            });
        }
    },

    storageAvailable: function (type) {
        try {
            var storage = window[type],
                x = '__storage_test__';
            storage.setItem(x, x);
            storage.removeItem(x);
            return true;
        }
        catch (e) {
            return false;
        }
    },

    registerOptimizationTest: function (optimizationTestNodeId, goalId, containerId, variants, ajax, variantsTraffic) {
        var selectedVariantId = null;
        if (wem.storageAvailable('sessionStorage')) {
            selectedVariantId = sessionStorage.getItem(optimizationTestNodeId);
        } else {
            selectedVariantId = wem.getCookie('selectedVariantId');
            if (selectedVariantId != null && selectedVariantId == '') {
                selectedVariantId = null;
            }
        }
        if (!(selectedVariantId && variants[selectedVariantId])) {
            var keys = Object.keys(variants);
            if (variantsTraffic) {
                var rand = 100 * Math.random() << 0;
                for (var nodeIdentifier in variantsTraffic) {
                    if ((rand -= variantsTraffic[nodeIdentifier]) < 0 && selectedVariantId == null) {
                        selectedVariantId = nodeIdentifier;
                    }
                }
            } else {
                selectedVariantId = keys[keys.length * Math.random() << 0];
            }
            if (wem.storageAvailable('sessionStorage')) {
                sessionStorage.setItem(optimizationTestNodeId, selectedVariantId);
            } else {
                wem.setCookie('selectedVariantId', selectedVariantId, 1);
            }
            wem.registerEvent({
                eventType: "optimizationTestEvent",
                scope: window.digitalData.scope,
                source: {
                    itemType: "page",
                    scope: window.digitalData.scope,
                    itemId: window.digitalData.page.pageInfo.pageID,
                    properties: window.digitalData.page
                },
                target: {
                    itemType: "optimizationTest",
                    scope: window.digitalData.scope,
                    itemId: optimizationTestNodeId,
                    properties: {
                        variantId: selectedVariantId,
                        goalId: goalId
                    }
                }
            });
        }
        //Trigger variant display event for optimization

        //Trigger variant display event for personalization
        var displayWemVariantEvent = document.createEvent("CustomEvent");
        displayWemVariantEvent.initCustomEvent('displayWemVariant', false, false, {
            type:"optimization",
            variantWrapperPath: variants[selectedVariantId].variantWrapperPath,
            variantPath: variants[selectedVariantId].variantPath,
            pagePath: variants[selectedVariantId].pagePath,
            variantWrapper: variants[selectedVariantId].variantWrapper,
            variant: selectedVariantId
        });
        document.dispatchEvent(displayWemVariantEvent);
        if (selectedVariantId) {
            if (ajax) {
                wem.loadContent(variants[selectedVariantId].content, containerId);
            } else {
                document.getElementById(variants[selectedVariantId].content).style.display = '';
            }
        }
    },

    extractFormData: function (form) {
        var params = {};
        for (var i=0; i < form.elements.length; i++) {
            var e = form.elements[i];
            if (typeof(e.name) != 'undefined') {
                switch (e.nodeName) {
                    case 'TEXTAREA':
                    case "INPUT":
                        switch (e.type) {
                            case 'checkbox':
                                var checkboxes = document.querySelectorAll('input[name="' + e.name + '"]');
                                if(checkboxes.length > 1) {
                                    if(!params[e.name]) {
                                        params[e.name] = [];
                                    }
                                    if(e.checked) {
                                        params[e.name].push(e.value);
                                    }

                                }
                                break;
                            case 'radio':
                                if (e.checked) {
                                    params[e.name] = e.value;
                                }
                                break;
                            default:
                                if (!e.value || e.value == "") {
                                    // ignore element if no value is provided
                                    break;
                                }
                                params[e.name] = e.value;
                        }
                        break;
                    case "SELECT":
                        if (e.options && e.options[e.selectedIndex]) {
                            if(e.multiple){
                                params[e.name] = [];
                                for (var i = 0; i < e.options.length; i++) {
                                    if(e.options[i].selected){
                                        params[e.name].push(e.options[i].value);
                                    }
                                }
                            }else{
                                params[e.name] = e.options[e.selectedIndex].value;
                            }
                        }
                        break;
                }
            }
        }
        return params;
    },

    onsucces: function (xhr, replaceIfExists) {
        if (!document.getElementById("contextjs")) {
            var oHead = document.getElementsByTagName('HEAD').item(0);
            var oScript = document.createElement("script");
            oScript.language = "javascript";
            oScript.type = "text/javascript";
            oScript.id = "contextjs";
            oScript.defer = true;
            oScript.text = xhr.responseText;
            oHead.appendChild(oScript);
        } else if (replaceIfExists) {
            var existingScript = document.getElementById("contextjs");
            existingScript.parentNode.removeChild(existingScript);
            var oHead = document.getElementsByTagName('HEAD').item(0);
            var oScript = document.createElement("script");
            oScript.language = "javascript";
            oScript.type = "text/javascript";
            oScript.id = "contextjs";
            oScript.defer = true;
            oScript.text = xhr.responseText;
            oHead.appendChild(oScript);
        }
    },

    executeFallback: function() {
        cxs = {};
        cxs.filteringResults = [];

        for (var index in window.digitalData.loadCallbacks) {
            window.digitalData.loadCallbacks[index]();
        }
    },

    loadContext: function (skipEvents, replaceContextIfExist) {
        var jsonData = {
            requiredProfileProperties: ['j:nodename'],
            source: {
                itemType: "page",
                scope: window.digitalData.scope,
                itemId: window.digitalData.page.pageInfo.pageID,
                properties:window.digitalData.page
            }
        };
        if (!skipEvents) {
            jsonData.events = window.digitalData.events
        }
        if (window.digitalData.filterCallback) {
            jsonData.filters = window.digitalData.filterCallback.map(function(x) {return x.filter})
        }

        wem.ajax({
            url: wem.contextServerUrl + '/context.js?sessionId=' + wem.sessionID,
            type: 'POST',
            async: true,
            contentType: "text/plain;charset=UTF-8", // Use text/plain to avoid CORS preflight
            jsonData: jsonData,
            dataType: 'text/javascript',
            success: function (xhr) {
                wem.onsucces(xhr, replaceContextIfExist)
            },
            error: wem.executeFallback
        });

        console.log("wem: context loading...");
    },

    loadPersonaContext: function (personaId, segmentOverrides, profilePropertiesOverrides, sessionPropertiesOverrides) {
        var jsonData = {
            source: {
                itemType: "page",
                scope: window.digitalData.scope,
                itemId: window.digitalData.page.pageInfo.pageID,
                properties: window.digitalData.page
            },
            requireSegments: true,
            requiredProfileProperties: ['*'],
            requiredSessionProperties: ['*'],
            events: window.digitalData.events,
            segmentOverrides: segmentOverrides,
            profilePropertiesOverrides: profilePropertiesOverrides,
            sessionPropertiesOverrides: sessionPropertiesOverrides
        };
        if (window.digitalData.filterCallback) {
            jsonData.filters = window.digitalData.filterCallback.map(function(x) {return x.filter})
        }

        wem.ajax({
            url: wem.contextServerUrl + '/context.js?'+ (wem.sessionID?'sessionId=' + wem.sessionID + '&' : '')+ "personaId=" + personaId,
            type: 'POST',
            async: true,
            contentType: "text/plain;charset=UTF-8", // Use text/plain to avoid CORS preflight
            jsonData: jsonData,
            dataType: 'text/javascript',
            success: function (xhr) {
                wem.onsucces(xhr, true)
            },
            error: wem.executeFallback
        });

        console.log("wem: persona context loading...");
    },

    formFactoryEventCallback: function(event) {
        console.log('Registring Form Factory event callback');
        window.ffCallbacks.registerCallback(event.detail, function (formData, formInfo) {
            if (wem.formNamesToWatch.indexOf(formInfo.formName) > -1) {
                console.info("wem: catching FF form " + event.detail.formName);
                var formEvent = {
                    scope: window.digitalData.scope,
                    eventType: "form",
                    source: {
                        itemType: "page",
                        scope: window.digitalData.scope,
                        itemId: window.digitalData.page.pageInfo.pageID,
                        properties: window.digitalData.page
                    },
                    target: {
                        itemType: "form",
                        scope: window.digitalData.scope,
                        itemId: formInfo.formName
                    },
                    properties: transformer(formData.resultData)
                };
                wem.sendFormEvent(formEvent, true, formInfo);
            }

            function transformer(data) {
                for (var key in data) {
                    if (data.hasOwnProperty(key)) {
                        var obj = data[key];
                        if (obj.rendererName && obj.rendererName === "country") {
                            //Grab country key only!!!
                            data[key] = obj.country.key;
                        }
                        if (obj.rendererName && obj.value) {
                            //Generic object with value key (rating etc.)
                            data[key] = obj.value;
                        }
                    }
                }
                return data;
            }
        });
    },

    formEventCallback: function (event) {
        console.log('Registering form event callback');
        var form = event.target;
        var formName = form.getAttribute("name") ? form.getAttribute("name") : form.getAttribute("id");
        if (formName && wem.formNamesToWatch.indexOf(formName) > -1) {
            console.info("wem: catching form " + formName);

            var eventCopy = document.createEvent('Event');
            // Define that the event name is 'build'.
            eventCopy.initEvent('submit', event.bubbles, event.cancelable);

            event.stopImmediatePropagation();
            event.preventDefault();

            var formEvent = {
                scope: window.digitalData.scope,
                eventType: "form",
                source:{
                    itemType: "page",
                    scope: window.digitalData.scope,
                    itemId: window.digitalData.page.pageInfo.pageID,
                    properties: window.digitalData.page
                },
                target: {
                    itemType:"form",
                    scope: window.digitalData.scope,
                    itemId:formName
                },
                properties: {}
            };
            // merge form properties with event properties
            var formData =  wem.extractFormData(form);
            for (var formInput in formData) {
                formEvent.properties[formInput]=formData[formInput];
            }

            wem.sendFormEvent(formEvent, false, form, eventCopy);
        }
    },

    sendFormEvent: function(formEvent, isFF, form, eventCopy) {
        cxs.collectEvent(formEvent,
            function () {
                console.info("wem: Form event successfully submitted.");
                if (!isFF) {
                    form.removeEventListener('submit', wem.formEventCallback, true);
                    form.dispatchEvent(eventCopy);
                    if (!eventCopy.defaultPrevented && !eventCopy.cancelBubble) {
                        form.submit();
                    }
                    form.addEventListener('submit', wem.formEventCallback, true);
                } else {
                    form.notifyFormFactoryOfCompletion("MFcallback");
                }
            },
            function (xhr) {
                console.log("wem: Error while collecting form event: " + xhr.status + " " + xhr.statusText);
                xhr.abort();
                if (!isFF) {
                    form.removeEventListener('submit', wem.formEventCallback, true);
                    form.dispatchEvent(eventCopy);
                    if (!eventCopy.defaultPrevented && !eventCopy.cancelBubble) {
                        form.submit();
                    }
                    form.addEventListener('submit', wem.formEventCallback, true);
                } else {
                    form.notifyFormFactoryOfCompletion("MFcallback");
                }
            }
        );
    },

    init: function(contextServerUrl, proxyServletUrl, isPreview, timeoutInMilliseconds, dxUsername) {
        wem.contextServerUrl = contextServerUrl;
        wem.proxyServletUrl = proxyServletUrl;
        wem.dxUsername = dxUsername;
        wem.timeoutInMilliseconds = timeoutInMilliseconds;
        wem.formNamesToWatch = [];
        wem.sessionID = wem.getCookie('wem-session-id');
        if (wem.sessionID == null) {
            console.log("SessionID is null !");
        } else if (wem.sessionId == '') {
            console.log("Empty sessionID, setting to null !");
            wem.sessionID = null;
        }

        if (isPreview) {
            // do not execute fallback for preview!
            return;
        }

        if (!navigator.cookieEnabled) {
            wem.executeFallback();
            return;
        }

        if (!wem.sessionID || wem.sessionID == '') {
            wem.executeFallback();
            return;
        }

        // this is the event to get form factory data when submitting
        document.addEventListener('ffFormReady', wem.formFactoryEventCallback);

        wem.registerEvent({
            eventType:"view",
            scope: window.digitalData.scope,
            source:{
                itemType: "site",
                scope: window.digitalData.scope,
                itemId: window.digitalData.site.siteInfo.siteID
            },
            target:{
                itemType: "page",
                scope: window.digitalData.scope,
                itemId: window.digitalData.page.pageInfo.pageID,
                properties: window.digitalData.page
            }
        });

        // process referrer if it exists
        var referrerURL = document.referrer;
        if (referrerURL) {
            // parse referrer URL
            var referrer = document.createElement('a');
            referrer.href = referrerURL;

            // only process referrer if it's not coming from the same site as the current page
            var local = document.createElement('a');
            local.href = document.URL;
            if (referrer.host !== local.host) {
                // get search element if it exists and extract search query if available
                var search = referrer.search;
                var query = undefined;
                if (search && search != "") {
                    // parse parameters
                    var queryParams = [], param;
                    var queryParamPairs = search.slice(1).split('&');
                    for (var i = 0; i < queryParamPairs.length; i++) {
                        param = queryParamPairs[i].split('=');
                        queryParams.push(param[0]);
                        queryParams[param[0]] = param[1];
                    }

                    // try to extract query: q is Google-like (most search engines), p is Yahoo
                    query = queryParams.q || queryParams.p;
                    query = decodeURIComponent(query).replace(/\+/g, ' ');
                }


                // register referrer event
                // Create deep copy of window.digitalData.page and add data to pageInfo sub object
                var properties = wem.extend(true, {}, window.digitalData.page);
                properties.pageInfo.referrerHost = referrer.host;
                properties.pageInfo.referrerQuery = query;
                
                wem.registerEvent({
                    eventType: "viewFromReferrer",
                    scope: window.digitalData.scope,
                    target: {
                        itemType: "page",
                        scope: window.digitalData.scope,
                        itemId: window.digitalData.page.pageInfo.pageID,
                        properties: properties
                    }
                });
            }
        }

        // Capture all submit at document level
        wem.registerCallbacks(function() {
            wem.setCookie("context-profile-id-copy", cxs.profileId);
            // process tracked events
            var videoNamesToWatch = [];
            if(cxs.trackedConditions && cxs.trackedConditions.length > 0){
                for (var i = 0; i < cxs.trackedConditions.length; i++) {
                    switch (cxs.trackedConditions[i].type) {
                        case "formEventCondition":
                            if(cxs.trackedConditions[i].parameterValues && cxs.trackedConditions[i].parameterValues.formId){
                                wem.formNamesToWatch.push(cxs.trackedConditions[i].parameterValues.formId)
                            }
                            break;
                        case "videoViewEventCondition":
                            if(cxs.trackedConditions[i].parameterValues && cxs.trackedConditions[i].parameterValues.videoId){
                                videoNamesToWatch.push(cxs.trackedConditions[i].parameterValues.videoId)
                            }
                            break;
                    }
                }
            }

            var forms = document.querySelectorAll('form');
            for (var formIndex = 0; formIndex < forms.length; formIndex++) {
                var form = forms.item(formIndex);
                var formName = form.getAttribute('name') ? form.getAttribute('name') : form.getAttribute('id');
                // test attribute data-form-id to not add a listener on FF form
                if (formName && wem.formNamesToWatch.indexOf(formName) > -1 && form.getAttribute('data-form-id') == null) {
                    // add submit listener on form that we need to watch only
                    console.info("wem: watching form " + formName);
                    form.addEventListener('submit', wem.formEventCallback, true);
                }
            }

            // register video events
            sendVideoEvent = function(event){
                console.info("wem: catching video event");
                var videoId = event.target.id;
                var videoEvent = {
                    scope: window.digitalData.scope,
                    eventType: "video",
                    source:{
                        itemType: "page",
                        scope: window.digitalData.scope,
                        itemId: window.digitalData.page.pageInfo.pageID,
                        properties: window.digitalData.page
                    },
                    target: {
                        itemType:"video",
                        scope: window.digitalData.scope,
                        itemId:videoId
                    },
                    properties: {
                        "event":event.type
                    }
                };

                cxs.collectEvent(videoEvent, function () {
                    console.log("wem: Video event successfully collected.");
                });
            };

            for (var j = 0; j < videoNamesToWatch.length; j++) {
                var videoName = videoNamesToWatch[j];
                var video = document.getElementById(videoName);
                if (video) {
                    video.addEventListener('play', sendVideoEvent);
                    video.addEventListener('ended', sendVideoEvent);
                    console.info("wem: watching video " + videoName);
                } else {
                    console.log("wem: unable to watch video " + videoName + ", video not found in the page");
                }
            }
        });

        document.addEventListener('DOMContentLoaded', function() {
            wem.registerCallbacks(function () {
                "use strict";
                wem.checkProfileValidity();
            });
            wem.loadContext();
        });

    },

    ajax: function(options) {
        var xhr = new XMLHttpRequest();
        if ('withCredentials' in xhr) {
            xhr.open(options.type, options.url, options.async);
            xhr.withCredentials = true;
        } else if (typeof XDomainRequest != 'undefined') {
            xhr = new XDomainRequest();
            xhr.open(options.type, options.url);
        }

        if (options.contentType) {
            xhr.setRequestHeader('Content-Type', options.contentType);
        }

        if (options.dataType) {
            xhr.setRequestHeader('Accept', options.dataType);
        }

        if (options.responseType) {
            xhr.responseType = options.responseType;
        }

        var wemExecuted = false;
        if (wem.timeoutInMilliseconds != -1) {
            setTimeout(function () {
                if (!wemExecuted) {
                    console.log("timeout, fallback personalization");
                    wemExecuted = true;
                    wem.executeFallback()
                }
            }, wem.timeoutInMilliseconds);
        }

        xhr.onreadystatechange = function () {
            if (!wemExecuted) {
                if (xhr.readyState == 4) {
                    if (xhr.status == 200 || xhr.status == 204 || xhr.status == 304) {
                        if (xhr.responseText != null) {
                            wemExecuted = true;
                            options.success(xhr);
                        }
                    } else {
                        options.error(xhr);
                        console.error('XML request error: ' + xhr.statusText + ' (' + xhr.status + ')');
                    }
                }
            }
        };

        if (options.jsonData) {
            xhr.send(JSON.stringify(options.jsonData));
        } else if (options.data) {
            xhr.send(options.data);
        } else {
            xhr.send();
        }
    },

    loadContent: function (url, parentSelector) {
        var xhr = new XMLHttpRequest();
        var finished = false;
        xhr.onabort = xhr.onerror = function xhrError() {
            finished = true;
        };

        xhr.onreadystatechange = function xhrStateChange() {
            if (xhr.readyState === 4 && !finished) {
                finished = true;
                try {
                    document.getElementById(parentSelector).innerHTML = xhr.responseXML.getElementsByTagName('body')[0].innerHTML;

                    var scriptsArray = document.getElementById(parentSelector).getElementsByTagName('script');
                    for (var i in scriptsArray) {
                        eval(scriptsArray[i].text);
                    }
                } catch (e) {
                    console.log(e);
                }
            }
        };

        xhr.open('GET', url);
        xhr.responseType = 'document';
        xhr.send();
    },

    getClosest: function(elem, selector) {
        // Element.matches() polyfill
        if (!Element.prototype.matches) {
            Element.prototype.matches =
                Element.prototype.matchesSelector ||
                Element.prototype.mozMatchesSelector ||
                Element.prototype.msMatchesSelector ||
                Element.prototype.oMatchesSelector ||
                Element.prototype.webkitMatchesSelector ||
                function(s) {
                    var matches = (this.document || this.ownerDocument).querySelectorAll(s),
                        i = matches.length;
                    while (--i >= 0 && matches.item(i) !== this) {}
                    return i > -1;
                };
        }

        // Get closest match
        for ( ; elem && elem !== document; elem = elem.parentNode ) {
            if (elem.matches( selector )) return elem;
        }

        return null;
    },

    extend: function() {
        // Variables
        var extended = {};
        var deep = false;
        var i = 0;
        var length = arguments.length;

        // Check if a deep merge
        if ( Object.prototype.toString.call( arguments[0] ) === '[object Boolean]' ) {
            deep = arguments[0];
            i++;
        }

        // Merge the object into the extended object
        var merge = function (obj) {
            for ( var prop in obj ) {
                if ( Object.prototype.hasOwnProperty.call( obj, prop ) ) {
                    // If deep merge and property is an object, merge properties
                    if ( deep && Object.prototype.toString.call(obj[prop]) === '[object Object]' ) {
                        extended[prop] = wem.extend( true, extended[prop], obj[prop] );
                    } else {
                        extended[prop] = obj[prop];
                    }
                }
            }
        };

        // Loop through each object and conduct a merge
        for ( ; i < length; i++ ) {
            var obj = arguments[i];
            merge(obj);
        }

        return extended;
    },

    generateGuid: function () {
        function s4() {
            return Math.floor((1 + Math.random()) * 0x10000)
                .toString(16)
                .substring(1);
        }

        return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
            s4() + '-' + s4() + s4() + s4();
    },

    invalidateSessionAndProfile: function () {
        "use strict";
        wem.sessionID = wem.generateGuid();
        wem.setCookie('wem-session-id',wem.sessionID + "-browser-generated", 1);
        wem.removeCookie("context-profile-id");
        wem.removeCookie("context-profile-id-copy");
        window.cxs = undefined;
    },

    checkProfileValidity : function () {
        "use strict";
        if (window.cxs) {
            // we will now check if the currently logged user in DX is equal to the username stored in the Unomi profile.
            // if not we will invalidate both the session and the profile and reload the context. This is related
            // to https://jira.jahia.org/browse/DMF-1468 and other reported issues about errors in profile switching
            // due mostly to provide in proper authentication tracking.
            if (cxs.profileProperties && cxs.profileProperties['j:nodename']) {
                var cxsDXNodeName = cxs.profileProperties['j:nodename'];
                if (cxsDXNodeName && wem.dxUsername != 'guest' && cxsDXNodeName != wem.dxUsername) {
                    console.log("Logged in DX user name ("+wem.dxUsername+") does not correspond with Unomi profile DX username (" + cxsDXNodeName + "). Invalidating session and profiles and reloading context.");
                    wem.invalidateSessionAndProfile();
                    wem.registerEvent({
                        eventType: "invalidProfileForDXUser",
                        scope: window.digitalData.scope,
                        target: {
                            itemType: "invalidProfileError",
                            scope: window.digitalData.scope,
                            itemId: cxsDXNodeName + "-" + wem.dxUsername,
                            properties: {
                                unomiNodeName : cxsDXNodeName,
                                dxUsername : wem.dxUsername
                            }
                        }
                    });
                    wem.loadContext();
                }
            }
        }
    }
};
